@@ -53,7 +53,7 @@ namespace Beagle {
 
 	public class Indexable : Versioned, IComparable {
 
-		static private bool Debug = false;
+		static private bool Debug = Beagle.Util.Debug.Enabled ("Indexable");
 
 		// This is the type of indexing operation represented by
 		// this Indexable object.  We default to Add, for historical
@@ -326,7 +326,7 @@ namespace Beagle {
 						Logger.Log.Debug ("Cleaning up {0} ({1})", contentUri.LocalPath, Uri);
 
 					try {
-						File.Delete (contentUri.LocalPath);
+						FileSystem.PosixDelete (contentUri.LocalPath);
 					} catch { 
 						// It might be gone already, so catch the exception.
 					}
@@ -339,7 +339,7 @@ namespace Beagle {
 						Logger.Log.Debug ("Cleaning up {0} ({1})", hotContentUri.LocalPath, Uri);
 
 					try {
-						File.Delete (hotContentUri.LocalPath);
+						FileSystem.PosixDelete (hotContentUri.LocalPath);
 					} catch {
 						// Ditto
 					}
@@ -635,7 +635,7 @@ namespace Beagle {
 			// Make sure the temporary file is only readable by the owner.
 			// FIXME: There is probably a race here.  Could some malicious program
 			// do something to the file between creation and the chmod?
-			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 256);
+			Mono.Unix.Native.Syscall.chmod (filename, Mono.Unix.Native.FilePermissions.S_IRUSR);
 
 			BufferedStream bufferedStream = new BufferedStream (fileStream);
 			StreamWriter writer = new StreamWriter (bufferedStream);
@@ -671,7 +671,7 @@ namespace Beagle {
 			// Make sure the temporary file is only readable by the owner.
 			// FIXME: There is probably a race here.  Could some malicious program
 			// do something to the file between creation and the chmod?
-			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 256);
+			Mono.Unix.Native.Syscall.chmod (filename, Mono.Unix.Native.FilePermissions.S_IRUSR);
 
 			BufferedStream bufferedStream = new BufferedStream (fileStream);
 
