@@ -58,7 +58,8 @@ namespace Beagle.Filters
 		public FilterDocbook ()
 		{
 			SnippetMode = false;
-			SetVersion (4);
+			SetVersion (5);
+			SetFileType ("documentation");
 		}
 
 		protected override void RegisterSupportedTypes ()
@@ -136,6 +137,9 @@ namespace Beagle.Filters
 							if (entry.Title == null)
 								entry.Title = reader.Value;
 						}
+					} else if (reader.Name == "keyword") {
+						reader.Read (); // read the text node
+						AddProperty (Property.NewKeyword ("dc:subject", reader.Value));
 					}
 					break;
 					
@@ -145,7 +149,8 @@ namespace Beagle.Filters
 						((DocbookEntry) entries_stack.Peek ()).Content.Append (reader.Value);
 
 					// Append text to the main indexable
-					AppendText (reader.Value);
+					else
+						AppendWord (reader.Value);
 					break;
 					
 				case XmlNodeType.EndElement:
@@ -158,9 +163,11 @@ namespace Beagle.Filters
 						if (entries_stack.Count > 0)
 							parent_entry = (DocbookEntry) entries_stack.Peek ();
 						
-						Indexable indexable = new Indexable (UriFu.PathToFileUri (String.Format ("{0}#{1}", base_path, entry.Id)));
+						Indexable indexable;
+						indexable = new Indexable (UriFu.AddFragment (Indexable.Uri, entry.Id, false));
 						indexable.HitType = "DocbookEntry";
 						indexable.MimeType = "text/x-docbook-entry";
+						indexable.AddProperty (Property.NewKeyword ("beagle:FileType", "documentation"));
 						indexable.Filtering = IndexableFiltering.AlreadyFiltered;
 
 						indexable.AddProperty (Property.NewUnsearched ("fixme:id", entry.Id));
@@ -184,8 +191,9 @@ namespace Beagle.Filters
 
 						StringReader content_reader = new StringReader (entry.Content.ToString ());
 						indexable.SetTextReader (content_reader);
-						
-						AddChildIndexable (indexable);
+						indexable.SetChildOf (this.Indexable);
+
+						AddIndexable (indexable);
 					}
 					break;
 				}
@@ -204,7 +212,8 @@ namespace Beagle.Filters
 			// successfull at all (unless we have a title, which
 			// means that it's actually a docbook file, just without
 			// sections.
-			if (ChildIndexables.Count == 0 && base_title == null) {
+			if (! HasGeneratedIndexable && base_title == null) {
+				Log.Error ("Probably not a docbook. Ignoring {0}!", base_path);
 				Error ();
 				return;
 			}
