@@ -37,7 +37,7 @@ namespace Beagle.Filters {
 
 	public class FilterTotem : Beagle.Daemon.Filter {
 
-		static bool Debug = false;
+		static bool Debug = Beagle.Util.Debug.Enabled ("FilterTotem");
 
 		public FilterTotem ()
 		{			
@@ -60,6 +60,7 @@ namespace Beagle.Filters {
 			pc.Arguments = new string [] { "totem-video-indexer", "--mimetype" };
 			pc.RedirectStandardOutput = true;
 			pc.RedirectStandardError = true;
+			pc.UseLangC = true;
 
 			try {
 				pc.Start ();
