@@ -49,61 +49,28 @@ namespace Beagle.Util {
 			}
 		}
 
-		// FIXME: These can all go away when we switch to generics.
-
-		// This is designed to be used instead of Array.IndexOf as it reduces
-		// boxing and is therefore more efficient.
-		public static int IndexOfByte (byte [] array, byte target, int start)
+		public static bool Equal (string[] arr1, string[] arr2)
 		{
-			for (int i = start; i < array.Length; ++i)
-				if (array [i] == target)
-					return i;
-			return -1;
-		}
+			if (arr1 == null && arr2 == null)
+				return true;
 
-		public static int IndexOfByte (byte [] array, byte target)
-		{
-			return IndexOfByte (array, target, 0);
-		}
+			if (arr1 == null || arr2 == null)
+				return false;
 
-		public static int IndexOfString (string [] array, string target, int start)
-		{
-			for (int i = start; i < array.Length; ++i)
-				if (array [i] == target)
-					return i;
-			return -1;
-		}
+			if (arr1.Length != arr2.Length)
+				return false;
 
-		public static int IndexOfString (string [] array, string target)
-		{
-			return IndexOfString (array, target, 0);
-		}
+			string[] new_1 = (string[]) arr1.Clone ();
+			string[] new_2 = (string[]) arr2.Clone ();
 
-		public static int IndexOfChar (string str, char target, int start)
-		{
-			for (int i = start; i < str.Length; ++i)
-				if (str [i] == target)
-					return i;
-			return -1;
-		}
+			Array.Sort (new_1);
+			Array.Sort (new_2);
 
-		public static int IndexOfChar (string str, char target)
-		{
-			return IndexOfChar (str, target, 0);
-		}
+			for (int i = 0; i < new_1.Length; ++ i)
+				if (new_1 [i] != new_2 [i])
+					return false;
 
-		public static int IndexOfChar (char [] array, char target, int start)
-		{
-			for (int i = start; i < array.Length; ++i)
-				if (array [i] == target)
-					return i;
-
-			return -1;
-		}
-
-		public static int IndexOfChar (char [] array, char target)
-		{
-			return IndexOfChar (array, target, 0);
+			return true;
 		}
 	}
 }
