@@ -114,12 +114,16 @@ namespace Beagle.Util {
 		[DllImport ("libbeagleglue.so")]
 		extern static unsafe int screensaver_glue_init ();
 
-		/*
-		 * BeagleDaemon needs to monitor screensaver status
-		 * for faster scheduling when user is idle.
-		 * IndexHelper does not need to monitor screensaver status.
-		 * XssInit is only called from the BeagleDaemon.
-		 */
+		/// <summary>
+		/// BeagleDaemon needs to monitor screensaver status
+		/// for faster scheduling when user is idle.
+		/// IndexHelper does not need to monitor screensaver status.
+		/// XssInit is only called from the BeagleDaemon.
+		///
+		/// </summary>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
 		public static bool XssInit ()
 		{
 			int has_xss = screensaver_glue_init ();
@@ -138,6 +142,12 @@ namespace Beagle.Util {
 			if (! use_screensaver)
 				return;
 
+			if (! Conf.Daemon.GetOption (Conf.Names.IndexFasterOnScreensaver, true)) {
+				cached_screensaver_running = false;
+				cached_screensaver_idle_time = 0;
+				return;
+			}
+
 			if ((DateTime.Now - screensaver_time).TotalSeconds < screensaver_poll_delay)
 				return;
 
@@ -168,8 +178,10 @@ namespace Beagle.Util {
 			}
 		}
 
-		// returns number of seconds since input was received
-		// from the user on any input device
+		/// <value>
+		///  returns number of seconds since input was received
+		/// from the user on any input device
+		/// </value>
 		public static double InputIdleTime {
 			get {
 				CheckScreenSaver ();
@@ -179,68 +191,6 @@ namespace Beagle.Util {
 
 		///////////////////////////////////////////////////////////////
 
-		const double acpi_poll_delay = 30;
-		const string ac_present_string = "on-line";
-
-		private static readonly string[] proc_ac_state_filenames = new string[] {
-			"/proc/acpi/ac_adapter/AC/state",
-			"/proc/acpi/ac_adapter/AC0/state",
-			"/proc/acpi/ac_adapter/ADp1/state",
-			"/proc/acpi/ac_adapter/ACAD/state"
-		};
-
-		private static string proc_ac_state_filename = null;
-		private static bool proc_ac_state_exists = true;
-		private static DateTime using_battery_time = DateTime.MinValue;
-		private static bool using_battery;
-
-		public static void CheckAcpi ()
-		{
-			if (! proc_ac_state_exists)
-				return;
-
-			if ((DateTime.Now - using_battery_time).TotalSeconds < acpi_poll_delay)
-				return;
-
-			if (proc_ac_state_filename == null) {
-				foreach (string s in proc_ac_state_filenames) {
-					if (File.Exists (s)) {
-						proc_ac_state_filename = s;
-						break;
-					}
-				}
-			}
-					
-			if (proc_ac_state_filename == null || ! File.Exists (proc_ac_state_filename)) {
-				proc_ac_state_exists = false;
-				using_battery = false;
-				return;
-			}
-
-			Stream stream = new FileStream (proc_ac_state_filename,
-							System.IO.FileMode.Open,
-							FileAccess.Read,
-							FileShare.Read);
-			TextReader reader = new StreamReader (stream);
-
-			string line = reader.ReadLine ();
-			using_battery = (line != null) && (line.IndexOf (ac_present_string) == -1);
-			
-			reader.Close ();
-			stream.Close ();
-
-			using_battery_time = DateTime.Now;
-		}
-
-		public static bool UsingBattery {
-			get { 
-				CheckAcpi ();
-				return using_battery;
-			}
-		}
-
-		///////////////////////////////////////////////////////////////
-
 		[DllImport ("libbeagleglue")]
 		extern static int get_vmsize ();
 
@@ -260,8 +210,8 @@ namespace Beagle.Util {
 			int vm_size = VmSize;
 			int vm_rss = VmRss;
 
-			Logger.Log.Debug ("Memory usage: VmSize={0:.0} MB, VmRSS={1:.0} MB,  GC.GetTotalMemory={2}",
-					  vm_size/1024.0, vm_rss/1024.0, GC.GetTotalMemory (false));
+			Logger.Log.Debug ("Memory usage: VmSize={0:.0} MB, VmRSS={1:.0} MB,  GC.GetTotalMemory={2} ({3} colls)",
+					  vm_size/1024.0, vm_rss/1024.0, GC.GetTotalMemory (false), GC.CollectionCount (2));
 		}
 
 		///////////////////////////////////////////////////////////////
@@ -401,19 +351,25 @@ namespace Beagle.Util {
 
 		///////////////////////////////////////////////////////////////
 
-		[DllImport("libc")]
-		private static extern int prctl (int option, byte [] arg2, ulong arg3, ulong arg4, ulong arg5);
-
 		// From /usr/include/linux/prctl.h
 		private const int PR_SET_NAME = 15;
 
+		[DllImport("libc")] // Linux
+		private static extern int prctl (int option, byte [] arg2, IntPtr arg3, IntPtr arg4, IntPtr arg5);
+
+		[DllImport ("libc")] // BSD
+		private static extern void setproctitle (byte [] fmt, byte [] str_arg);
+
 		public static void SetProcessName(string name)
 		{
 #if OS_LINUX
-			if (prctl (PR_SET_NAME, Encoding.ASCII.GetBytes (name + '\0'), 0, 0, 0) < 0) {
+			if (prctl (PR_SET_NAME, Encoding.ASCII.GetBytes (name + '\0'), 
+				   IntPtr.Zero, IntPtr.Zero, IntPtr.Zero) < 0) {
 				Logger.Log.Warn ("Couldn't set process name to '{0}': {1}", name,
 						 Mono.Unix.Native.Stdlib.GetLastError ());
 			}
+#elif OS_FREEBSD
+			setproctitle (Encoding.ASCII.GetBytes ("%s\0"), Encoding.ASCII.GetBytes (name + "\0"));
 #endif
 		}
 
@@ -441,13 +397,12 @@ namespace Beagle.Util {
 		public static void Main ()
 		{
 			while (true) {
-				Console.WriteLine ("{0} {1} {2} {3} {4} {5} {6} {7}",
+				Console.WriteLine ("{0} {1} {2} {3} {4} {5} {6}",
 						   LoadAverageOneMinute,
 						   LoadAverageFiveMinute,
 						   LoadAverageFifteenMinute,
 						   ScreenSaverRunning,
 						   InputIdleTime,
-						   UsingBattery,
 						   DiskStatsReadReqs,
 						   VmSize);
 				System.Threading.Thread.Sleep (1000);
