@@ -117,7 +117,9 @@ class DumpIndexTool {
 					internal_uri = UriFu.UriToEscapedString (hit.ParentUri);
 					
 					hit.ParentUri = UriFu.PathToFileUri (path);
-					hit.Uri = UriFu.PathToFileUri (path, hit.Uri.Fragment);
+					hit.Uri = UriFu.AddFragment (UriFu.PathToFileUri (path),
+								     hit.Uri.Fragment,
+								     true);
 				} else {
 					internal_uri = UriFu.UriToEscapedString (hit.Uri);
 
@@ -168,8 +170,8 @@ class DumpIndexTool {
 				char [] legend = new char [4];
 
 				legend [0] = prop.IsMutable  ? 'm' : ' ';
-				legend [1] = prop.IsStored   ? 's' : ' ';
-				legend [2] = prop.IsSearched ? 's' : ' ';
+				legend [1] = prop.IsSearched ? 's' : ' ';
+				legend [2] = prop.IsPersistent ? 'p' : ' ';
 				legend [3] = prop.Type == PropertyType.Text ? 't' : ' ';
 
 				Console.WriteLine ("  Prop: [{0}] {1} = '{2}'", new String (legend), prop.Key, prop.Value);
@@ -205,7 +207,7 @@ class DumpIndexTool {
 				int freq;
 				freq = term_enum.DocFreq ();
 
-				Console.WriteLine ("{0} {1} {2}", index_name, term_enum.Term ().Text (), freq);
+				Console.WriteLine ("{0} '{1}' {2}", index_name, term_enum.Term ().Text (), freq);
 
 				// FIXME: spew these as a count
 				++distinct_term_count;
@@ -222,6 +224,31 @@ class DumpIndexTool {
 		Console.WriteLine ();
 	}
 
+	// Dump the fields: we do this via direct Lucene access.
+	static void DumpOneIndex_Fields (string index_name)
+	{
+		LuceneQueryingDriver driver;
+		driver = new LuceneQueryingDriver (index_name, -1, true);
+		
+		IndexReader reader;
+		reader = IndexReader.Open (driver.PrimaryStore);
+
+		Console.WriteLine ("  -- Primary Index --");
+		foreach (DictionaryEntry fi in reader.GetFieldNames (IndexReader.FieldOption.ALL))
+			Console.WriteLine ("- [{0}]", fi.Key);
+		reader.Close ();
+
+		reader = IndexReader.Open (driver.SecondaryStore);
+		if (reader.MaxDoc () != 0) {
+			Console.WriteLine ("\n  -- Secondary Index --");
+			foreach (DictionaryEntry fi in reader.GetFieldNames (IndexReader.FieldOption.ALL))
+				Console.WriteLine ("- [{0}]", fi.Key);
+			reader.Close ();
+		}
+
+		Console.WriteLine ();
+	}
+
 	/////////////////////////////////////////////////////////
 		
 	public class IndexInfo : IComparable {
@@ -242,14 +269,16 @@ class DumpIndexTool {
 
 	static void DumpIndexInformation (ArrayList indexes, ArrayList uris, bool show_properties, bool show_counts)
 	{
+		LuceneCommon.DumpIndexMode =  true;
+
 		foreach (IndexInfo info in indexes)
 			info.Count = DumpOneIndex_Metadata (info.Name, uris, show_properties);
 
 		if (show_properties) {
 			Console.WriteLine ("LEGEND:");
 			Console.WriteLine ("  m - mutable");
-			Console.WriteLine ("  s - stored");
 			Console.WriteLine ("  s - searched");
+			Console.WriteLine ("  p - persistent");
 			Console.WriteLine ("  t - tokenized");
 		}
 
@@ -268,6 +297,12 @@ class DumpIndexTool {
 			DumpOneIndex_TermFrequencies (info.Name);
 	}
 
+	static void DumpIndexFields (ArrayList indexes)
+	{
+		foreach (IndexInfo info in indexes)
+			DumpOneIndex_Fields (info.Name);
+	}
+
 	/////////////////////////////////////////////////////////
 
 	static void PrintUsage ()
@@ -282,6 +317,7 @@ Usage: beagle-dump-index [options] [[file or URI to match] ...]
   --uris                   Dump all Uris (default)
   --properties             Dump all properties
   --term-frequencies       Dump term frequencies
+  --fields                 Dump all fields
 
   --show-counts            Show index count totals (default)
   --hide-counts            Hide index count totals
@@ -303,7 +339,8 @@ Usage: beagle-dump-index [options] [[file or URI to match] ...]
 	enum Mode {
 		Uris,
 		Properties,
-		TermFrequencies
+		TermFrequencies,
+		Fields
 	}
 
 	static void Main (string [] args)
@@ -345,6 +382,10 @@ Usage: beagle-dump-index [options] [[file or URI to match] ...]
 				show_counts = false;
 				break;
 
+			case "--fields":
+				mode = Mode.Fields;
+				break;
+
 			default:
 				if (arg.StartsWith ("--indexdir="))
 					index_dirs.Add (arg.Remove (0, 11));
@@ -354,9 +395,9 @@ Usage: beagle-dump-index [options] [[file or URI to match] ...]
 					Uri uri;
 					
 					try {
-						uri = new Uri (arg);
+						uri = UriFu.UserUritoEscapedUri (arg);
 					} catch (UriFormatException) {
-						uri = new Uri (Path.GetFullPath (arg));
+						uri = UriFu.PathToFileUri (arg);
 					}
 
 					uris.Add (uri);
@@ -365,8 +406,8 @@ Usage: beagle-dump-index [options] [[file or URI to match] ...]
 			}
 		}
 
-		if (uris.Count > 0 && mode == Mode.TermFrequencies) {
-			Console.WriteLine ("ERROR: --term-frequencies doesn't make sense with files or URIs.");
+		if (uris.Count > 0 && (mode == Mode.TermFrequencies || mode == Mode.Fields)) {
+			Console.WriteLine ("ERROR: --term-frequencies and --fields do not make sense with files or URIs.");
 			Environment.Exit (1);
 		}
 
@@ -396,7 +437,9 @@ Usage: beagle-dump-index [options] [[file or URI to match] ...]
 
 		if (mode == Mode.Uris || mode == Mode.Properties)
 			DumpIndexInformation (indexes, uris, mode == Mode.Properties, show_counts);
-		else
+		else if (mode == Mode.TermFrequencies)
 			DumpIndexTermFreqs (indexes);
+		else if (mode == Mode.Fields)
+			DumpIndexFields (indexes);
 	}
 }
