@@ -40,7 +40,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 	[QueryableFlavor (Name="Files", Domain=QueryDomain.Local | QueryDomain.Neighborhood, RequireInotify=false)]
 	public class FileSystemQueryable : LuceneQueryable {
 
-		static internal bool Debug = false;
+		static internal bool Debug = Beagle.Util.Debug.Enabled ("FSQ");
 
 		// History:
 		// 1: Initially set to force a reindex due to NameIndex changes.
@@ -52,7 +52,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		//    offset the change in LuceneCommon.cs
 		// 6: Store beagle:FileType property denoting type of file like document, source, music etc.
 		// 7: Store filesize as fixme:filesize
-		const int MINOR_VERSION = 7;
+		// 8: Force re-index since now files whose EA gets removed wont get stored multiple times.
+		const int MINOR_VERSION = 8;
 
 		private object big_lock = new object ();
 
@@ -84,9 +85,6 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 		public FileSystemQueryable () : base ("FileSystemIndex", MINOR_VERSION)
 		{
-			if (! Debug)
-				Debug = (Environment.GetEnvironmentVariable ("BEAGLE_DEBUG_FSQ") != null);
-
 			// Set up our event backend
 			if (Inotify.Enabled) {
                                 Logger.Log.Debug ("Starting Inotify FSQ file event backend");
@@ -1096,7 +1094,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			if (attr == null) {
 				if (Debug)
 					Logger.Log.Debug ("*** Index it: File has no attributes");
-				id = uid_manager.ReadOrCreateNewId (path);
+				id = uid_manager.ReadOrCreateNewId (dir, name);
 				return RequiredAction.Index;
 			}
 
@@ -1168,7 +1166,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				if (path != last_known_path) {
 					if (Debug)
 						Logger.Log.Debug ("*** Name has also changed, assigning new unique id");
-					id = uid_manager.ReadOrCreateNewId (path);
+					id = uid_manager.CreateNewId (path);
 				}
 				
 				return RequiredAction.Index;
