@@ -2,6 +2,7 @@
 // Filter.cs
 //
 // Copyright (C) 2004 Novell, Inc.
+// Copyright (C) 2006-2007 Debajyoti Bera <dbera.web@gmail.com>
 //
 
 //
@@ -27,6 +28,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Text;
 using System.Reflection;
@@ -37,11 +39,13 @@ namespace Beagle.Daemon {
 
 	public class Filter {
 
-		static private bool Debug = false;
-		// Lucene fields allow a maximum of 10000 words
-		// Some of the words will be stop words... so a failsafe maximum of 40000 words
-		// Dont accept more words than that
-		const int MAXWORDS = 40000; // Lucene.Net.Index.IndexWriter.DEFAULT_MAX_FIELD_LENGTH * 4
+		static private bool Debug = Beagle.Util.Debug.Enabled ("Filter");
+
+		// Number of characters Filters should return in each DoPull()
+		// Optimization hint. -1 means unlimited.
+		const int DEFAULT_CHARS_TO_PULL = 2048;
+		int chars_to_pull = -1;
+		int chars_added = 0;
 
 		// Derived classes always must have a constructor that
 		// takes no arguments.
@@ -55,7 +59,8 @@ namespace Beagle.Daemon {
 		{
 			// Add flavor only when called from RegisterSupportedTypes
 			if (supported_flavors == null)
-				return;
+				throw new Exception ("AddSupportedFlavor() should be only called from RegisterSupportedTypes()");
+
 			supported_flavors.Add (flavor);
 		}
 
@@ -96,7 +101,7 @@ namespace Beagle.Daemon {
 				throw new Exception (msg);
 			}
 
-			if (version != -1) {
+			if (v <= version) {
 				string msg;
 				msg = String.Format ("Attempt to re-set version from {0} to {1} on Filter {2}", version, v, Name);
 				throw new Exception (msg);
@@ -111,8 +116,8 @@ namespace Beagle.Daemon {
 
 		private string this_mime_type = null;
 		private string this_extension = null;
-		private ArrayList indexable_properties = null;
-		private DateTime timestamp = DateTime.MinValue;
+		private string this_file_type = null;
+		private Indexable indexable = null;
 
 		public string MimeType {
 			get { return this_mime_type; }
@@ -124,41 +129,35 @@ namespace Beagle.Daemon {
 			set { this_extension = value; }
 		}
 
-		// Allow the filter to access the properties
-		// set by indexable
-		public ArrayList IndexableProperties {
-			get { return indexable_properties; }
-			set { indexable_properties = value; }
-		}
-
-		// Allow the filter to access the timestamp,
-		// sometime filters know better
-		public DateTime Timestamp {
-			get { return timestamp; }
-			set { timestamp = value; }
-		}
-		
-		//////////////////////////
-		
-		private bool crawl_mode = false;
-
-		public void EnableCrawlMode ()
+		/// <summary>
+		///  Filter may set the filetype to document, source, music etc.
+		/// Use lower case for file_type
+		/// </summary>
+		/// <param name="file_type">
+		/// A <see cref="System.String"/>
+		/// </param>
+		protected void SetFileType (string file_type)
 		{
-			crawl_mode = true;
+			this_file_type = file_type;
+		}
+
+		public string FileType {
+			get { return this_file_type; }
+		}
+
+		public Indexable Indexable {
+			get { return indexable; }
+			set { indexable = value; }
 		}
 		
-		protected bool CrawlMode {
-			get { return crawl_mode; }
-		}
-
-		//////////////////////////
-
-		// Filters which deal with big files, and that don't need
-		// to read in whole files may want to set this to false
-		// to avoid wasting cycles in disk wait.
+		
 
 		private bool preload = true;
-
+		/// <value>
+		///  Filters which deal with big files, and that don't need
+		/// to read in whole files may want to set this to false
+		/// to avoid wasting cycles in disk wait.
+		/// </value>
 		protected bool PreLoad {
 			get { return preload; }
 			set { preload = value; }
@@ -223,25 +222,25 @@ namespace Beagle.Daemon {
 
 		//////////////////////////
 
+		// Various methods for filters to report extracted text
+		// All the methods return true/false saying whether more text is requested
+		// Filters can choose to ignore the return value
+
+		// Use a shared text builder
+		private StringBuilder text_builder = new StringBuilder (DEFAULT_CHARS_TO_PULL);
 		private ArrayList textPool;
 		private ArrayList hotPool;
-		private ArrayList propertyPool;
 
-		private int word_count = 0;
-		private int hotword_count = 0;
-
-		protected bool AllowMoreWords ()
-		{
-			return (word_count < MAXWORDS);
-		}
-		
-		private bool last_was_structural_break = true;
+		private bool last_was_structural_break = false;
 		const string WHITESPACE = " ";
+		const string NEWLINE = "\n";
 
-		/* Append text to the textpool. If IsHot is true, then also add to the hottext pool.
-		 * Handles null str.
-		 */
-		public int AppendText (string str)
+		
+		/// <summary>
+		///  Append text to the textpool. If IsHot is true, then also add to the hottext pool.
+		///  Handles null str.
+		/// </summary>
+		public bool AppendText (string str)
 		{
 			if (Debug)
 				Logger.Log.Debug ("AppendText (\"{0}\")", str);
@@ -267,64 +266,53 @@ namespace Beagle.Daemon {
 		 * of AppendText ()
 		 */
 
-		public int AppendText (string str, string strHot)
+		public bool AppendText (string str, string strHot)
 		{
-			int num_words = 0;
-
 			if (Debug)
 				Logger.Log.Debug ("AppendText (\"{0}, {1}\")", str, strHot);
 
 			if (IsFrozen)
-				return 0;
+				return true;
 
-			if (word_count < MAXWORDS && ! string.IsNullOrEmpty (str)) {
-				string[] lines;
+			if (! string.IsNullOrEmpty (str)) {
+				last_was_structural_break = false;
 
-				// Avoid unnecessary allocation of a string
-				// FIXME: Handle \r, \r\n cases.
-				if (str.IndexOf ('\n') > -1) {
-					lines = str.Split ('\n'); 
-					foreach (string line in lines) {
-						if (line.Length > 0) {
-							ReallyAppendText (line);
-							AppendStructuralBreak ();
-						}
-					}
-				} else 
-					ReallyAppendText (str);
+				// If the filter is not storing snippets, there
+				// is no need to break strings into different lines.
+				ReallyAddText (str);
+				if (snippetWriter != null)
+					// Previously, str was stripped into newlines and
+					// each line was added individually. Effectively
+					// that was removing blank lines from snippets.
+					// Now blank lines are checked while reading from
+					// the cache. No processing done here. TextCache files
+					// are stored compressed so there would not be significant
+					// storage overhead in doing this.
+					snippetWriter.Write (str);
 
-				num_words = StringFu.CountWords (str, 3, -1);
-				word_count += num_words;
+				return UpdateCharsAdded (str.Length);
+			} else {
+				return true;
 			}
 
-			if (hotword_count < MAXWORDS && ! string.IsNullOrEmpty (strHot)) {
-				ReallyAppendHotText (strHot);
-				hotword_count += StringFu.CountWords (strHot, 3, -1);
+			/* FIXME: Disable HotText for now. Enable it when we start using it for query.
+			if (! string.IsNullOrEmpty (strHot)) {
+				hotPool.Add (strHot);
+				hotPool.Add (WHITESPACE);
 			}
-
-			return num_words;
-		}
-		
-		// strHot may not be null or empty - checked before
-		private void ReallyAppendHotText (string strHot)
-		{
-			hotPool.Add (strHot.Trim());
-			hotPool.Add (WHITESPACE);
+			*/
 		}
 
-		// str may not be null or empty - checked before
-		private void ReallyAppendText (string str)
-		{
-			textPool.Add (str);
-
-			if (snippetWriter != null)
-				snippetWriter.Write (str);
-
-			last_was_structural_break = false;
-		}
-
-		// Add a word followed by a whitespace. word may not be whitespace or newline.
-		public int AppendWord (string word)
+		/// <summary>
+		/// Add a word followed by a whitespace. word may not be whitespace or newline.
+		/// </summary>
+		/// <param name="word">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public bool AppendWord (string word)
 		{
 			if (Debug)
 				Logger.Log.Debug ("AppendWord (\"{0}\")", word);
@@ -332,8 +320,16 @@ namespace Beagle.Daemon {
 			return AppendWords (word, false);
 		}
 
-		// Add a line followed by a newline.
-		public int AppendLine (string line)
+		/// <summary>
+		/// Add a line followed by a newline.
+		/// </summary>
+		/// <param name="line">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public bool AppendLine (string line)
 		{
 			if (Debug)
 				Logger.Log.Debug ("AppendLine (\"{0}\")", line);
@@ -341,88 +337,190 @@ namespace Beagle.Daemon {
 			return AppendWords (line, true);
 		}
 
-		private int AppendWords (string words, bool is_line)
+		private bool AppendWords (string words, bool is_line)
 		{
 			if (IsFrozen || string.IsNullOrEmpty (words))
-				return 0;
+				return true;
 
+			/* FIXME: Disable hottext for now. Enable it when LuceneCommon starts using it for query.
 			if (IsHot) {
 				hotPool.Add (words);
 				hotPool.Add (WHITESPACE);
-				hotword_count += StringFu.CountWords (words, 3, -1);
 			}
+			*/
 
-			textPool.Add (words);
+			ReallyAddText (words);
+			if (is_line) {
+				text_builder.Append (NEWLINE);
+				last_was_structural_break = true;
+			} else
+				text_builder.Append (WHITESPACE);
 
-			if (snippetWriter != null)
+			if (snippetWriter != null) {
 				snippetWriter.Write (words);
 
-			if (is_line)
-				AppendStructuralBreak ();
-			else
-				AppendWhiteSpace ();
+				if (is_line) {
+					snippetWriter.WriteLine ();
+				} else {
+					snippetWriter.Write (WHITESPACE);
+				}
+			}
 
-			int num_words = StringFu.CountWords (words, 3, -1);
-			word_count += num_words;
-			return num_words;
+			return UpdateCharsAdded (words.Length + 1);
 		}
 
-		/*
-		 * Adds whitespace to the textpool.
-		 */
-		public void AppendWhiteSpace ()
+		/// <summary>
+		/// Does not check for structural breaks
+		/// </summary>
+		/// <param name="buffer">
+		/// A <see cref="System.Char"/>
+		/// </param>
+		/// <param name="index">
+		/// A <see cref="System.Int32"/>
+		/// </param>
+		/// <param name="count">
+		/// A <see cref="System.Int32"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public bool AppendChars (char[] buffer, int index, int count)
 		{
-			if (last_was_structural_break)
-				return;
+			if (Debug)
+				Logger.Log.Debug ("AppendChars ()");
 
+			ReallyAddText (buffer, index, count);
+
+			if (snippetWriter != null)
+				snippetWriter.Write (buffer, index, count);
+
+			return UpdateCharsAdded (count);
+		}
+
+		
+		 
+		/// <summary>
+		/// Adds whitespace to the textpool.
+		/// </summary>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public bool AppendWhiteSpace ()
+		{
 			if (Debug)
 				Logger.Log.Debug ("AppendWhiteSpace ()");
 
-			if (NeedsWhiteSpace (textPool)) {
-				textPool.Add (WHITESPACE);
-				if (snippetWriter != null)
-					snippetWriter.Write (WHITESPACE);
-				last_was_structural_break = false;
-			}
-		}
-
-		/*
-		 * Creates a new paragraph. Mainly useful for storing cached contents.
-		 */
-		public void AppendStructuralBreak ()
-		{
-			if (snippetWriter != null && ! last_was_structural_break) {
-				snippetWriter.WriteLine ();
-				last_was_structural_break = true;
-			}
-
-			// When adding a "newline" to the textCache, we need to 
-			// append a "Whitespace" to the text pool.
-			if (NeedsWhiteSpace (textPool))
-				textPool.Add (WHITESPACE);
-		}
-
-		private bool NeedsWhiteSpace (ArrayList array)
-		{
-			if (array.Count == 0)
+			if (last_was_structural_break)
 				return true;
-			
-			string last = (string) array [array.Count-1];
-			if (last.Length > 0
-			    && char.IsWhiteSpace (last [last.Length-1]))
+
+			last_was_structural_break = false;
+
+			text_builder.Append (WHITESPACE);
+
+			if (snippetWriter != null)
+				snippetWriter.Write (WHITESPACE);
+
+			return UpdateCharsAdded (1);
+		}
+
+		
+		
+		/// <summary>
+		/// Creates a new paragraph. Mainly useful for storing cached contents.
+		/// </summary>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public bool AppendStructuralBreak ()
+		{
+			if (Debug)
+				Logger.Log.Debug ("AppendStructuralBreak ()");
+
+			if (last_was_structural_break)
+				return true;
+
+			last_was_structural_break = true;
+
+			text_builder.Append (NEWLINE);
+
+			if (snippetWriter != null)
+				snippetWriter.WriteLine ();
+
+			return UpdateCharsAdded (1);
+		}
+
+#if ENABLE_RDF_ADAPTER
+		private IList<string> links_list = new List<string> ();
+
+		public IList<string> Links {
+			get { return links_list; }
+		}
+#endif
+
+		public void AddLink (string link)
+		{
+#if ENABLE_RDF_ADAPTER
+			links_list.Add (link);
+#endif
+		}
+
+		//private bool NeedsWhiteSpace (ArrayList array)
+		//{
+		//	if (array.Count == 0)
+		//		return true;
+		//	
+		//	string last = (string) array [array.Count-1];
+		//	if (last.Length > 0
+		//	    && char.IsWhiteSpace (last [last.Length-1]))
+		//		return false;
+
+		//	return true;
+		//}
+
+		private void ReallyAddText (string str)
+		{
+			text_builder.Append (str);
+
+			// Store text in blocks of whatever Pull() specified or DEFAULT_CHARS_TO_PULL o/w
+			if (text_builder.Length >= (chars_to_pull > 0 ? chars_to_pull : DEFAULT_CHARS_TO_PULL)) {
+				// FIXME: If text-cache needs to be cleaned up (i.e. whitespaces merged and blank
+				// lines removed, then process text_builder here before adding it to textPool
+				// Also, add to snippet-writer from here
+				textPool.Add (text_builder.ToString ());
+				text_builder.Length = 0;
+			}
+		}	
+
+		private void ReallyAddText (char[] buffer, int index, int count)
+		{
+			text_builder.Append (buffer, index, count);
+
+			// Store text in blocks of whatever Pull() specified or DEFAULT_CHARS_TO_PULL o/w
+			if (text_builder.Length >= (chars_to_pull > 0 ? chars_to_pull : DEFAULT_CHARS_TO_PULL)) {
+				// FIXME: If text-cache needs to be cleaned up (i.e. whitespaces merged and blank
+				// lines removed, then process text_builder here before adding it to textPool
+				// Also, add to snippet-writer from here
+				textPool.Add (text_builder.ToString ());
+				text_builder.Length = 0;
+			}
+		}	
+
+		private bool UpdateCharsAdded (int append_chars_added)
+		{
+			chars_added += append_chars_added;
+
+			if (chars_to_pull >= 0 && chars_added >= chars_to_pull) {
 				return false;
+			}
 
 			return true;
 		}
+		
+		//////////////////////////
 
-		/*
-		 * Adds property prop.
-		 * prop can be null or can have null value; in both cases nothing is added.
-		 */
 		public void AddProperty (Property prop)
 		{
-			if (prop != null && ! string.IsNullOrEmpty (prop.Value))
-				propertyPool.Add (prop);
+			this.Indexable.AddProperty (prop);
 		}
 
 		//////////////////////////
@@ -435,6 +533,10 @@ namespace Beagle.Daemon {
 		
 		protected void Finished ()
 		{
+			textPool.Add (text_builder.ToString ());
+			text_builder.Length = 0;
+
+			Close (); // Close streams and clear tmpfiles when done
 			isFinished = true;
 		}
 
@@ -446,7 +548,11 @@ namespace Beagle.Daemon {
 
 		protected void Error ()
 		{
-			Cleanup (); // force the clean-up of temporary files on an error
+			// Add whatever was reported till error
+			textPool.Add (text_builder.ToString ());
+			text_builder.Length = 0;
+
+			Close (); // Close streams and clear tmpfiles on error
 			has_error = true;
 		}
 
@@ -489,7 +595,7 @@ namespace Beagle.Daemon {
 
 		private string tempFile = null;
 		private FileSystemInfo currentInfo = null;
-		private FileStream currentStream = null;
+		private Stream currentStream = null;
 		private StreamReader currentReader = null;
 
 		public bool Open (TextReader reader)
@@ -561,12 +667,58 @@ namespace Beagle.Daemon {
 			return Open (new FileInfo (tempFile));
 		}
 
+		/// <summary>
+		/// This will throw an exception; callers should catch it and appropriately
+		/// display the error message showing the filename etc.
+		/// </summary>
+		/// <param name="stream">
+		/// A <see cref="Stream"/>
+		/// </param>
+		/// <param name="store_tempfile">
+		/// A <see cref="System.Boolean"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public bool Open (Stream stream, bool store_tempfile)
+		{
+			if (store_tempfile)
+				return Open (stream);
+
+			currentStream = stream;
+
+			isFinished = false;
+			if (textPool == null)
+				textPool = new ArrayList ();
+			if (hotPool == null)
+				hotPool = new ArrayList ();
+			text_builder.Length = 0;
+
+			// Note: No call to DoOpen ()
+			// Some filter might need a DoOpen(),
+			// but then they should be stored in a tempfile.
+
+			DoPullProperties ();
+			
+			if (IsFinished) 
+				return true;
+			else if (HasError)
+				return false;
+			
+			DoPullSetup ();
+			
+			if (HasError)
+				return false;				
+
+			return true;
+		}
+
 		public bool Open (FileSystemInfo info)
 		{
 			isFinished = false;
 			textPool = new ArrayList ();
 			hotPool = new ArrayList ();
-			propertyPool = new ArrayList ();
+			text_builder.Length = 0;
 
 			currentInfo = info;
 
@@ -580,11 +732,11 @@ namespace Beagle.Daemon {
 				if (preload) {
 					// Our default assumption is sequential reads.
 					// FIXME: Is this the right thing to do here?
-					FileAdvise.IncreaseReadAhead (currentStream);
+					FileAdvise.IncreaseReadAhead ((FileStream) currentStream);
 				
 					// Give the OS a hint that we will be reading this
 					// file soon.
-					FileAdvise.PreLoad (currentStream);				
+					FileAdvise.PreLoad ((FileStream) currentStream);				
 				}
 			}
 
@@ -595,25 +747,13 @@ namespace Beagle.Daemon {
 					return true;
 				else if (HasError)
 					return false;
-				
-				DoPullProperties ();
-				
-				if (IsFinished) 
-					return true;
-				else if (HasError)
-					return false;
-				
-				DoPullSetup ();
-				
-				if (HasError)
-					return false;				
+				    
+				return Open (currentStream, false);
 			} catch (Exception e) {
 				Log.Warn (e, "Unable to filter {0}:", info.FullName);
 				Cleanup (); // clean up temporary files on an exception
 				return false;
 			}
-
-			return true;
 		}
 
 		public bool Open (string path)
@@ -649,23 +789,31 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		// Should only be called for Text and NOT for hottext
+		// hottext should already be in place by the time Text pull is over
 		private bool Pull ()
 		{
-			if (IsFinished || HasError) {
-				Close ();
-				return false;
+			while (! IsFinished && ! HasError) {
+				DoPull ();
+				if (chars_to_pull >= 0 && chars_added >= chars_to_pull) {
+					chars_added = 0;
+					break;
+				}
 			}
 
-			DoPull ();
-
-			if (HasError)
+			if (IsFinished || HasError)
 				return false;
 
 			return true;
 		}
 
+		private bool closed = false;
+
 		private void Close ()
 		{
+			if (closed)
+				return;
+
 			Cleanup ();
 
 			DoClose ();
@@ -676,8 +824,8 @@ namespace Beagle.Daemon {
 			if (currentStream != null) {
 				// When crawling, give the OS a hint that we don't
 				// need to keep this file around in the page cache.
-				if (CrawlMode)
-					FileAdvise.FlushCache (currentStream);
+				if (indexable.FlushBufferCache && currentStream is FileStream)
+					FileAdvise.FlushCache ((FileStream) currentStream);
 
 				currentStream.Close ();
 				currentStream = null;
@@ -685,6 +833,8 @@ namespace Beagle.Daemon {
 
 			if (snippetWriter != null)
 				snippetWriter.Close ();
+
+			closed = true;
 		}
 
 		public void Cleanup ()
@@ -721,9 +871,11 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
-		private bool PullTextCarefully (ArrayList array, StringBuilder sb, bool is_hot)
+		private bool PullTextCarefully (ArrayList array, StringBuilder sb, int chars_to_pull, bool is_hot)
 		{
 			bool pulled = false;
+			this.chars_to_pull = chars_to_pull;
+
 			try {
 				pulled = PullFromArray (array, sb, is_hot);
 			} catch (Exception ex) {
@@ -733,19 +885,22 @@ namespace Beagle.Daemon {
 			return pulled;
 		}
 
-		private bool PullText (StringBuilder sb)
+		// chars_to_pull is a hint about the number of chars to pull before PullText is called again
+		private bool PullText (StringBuilder sb, int chars_to_pull)
 		{
-			return PullTextCarefully (textPool, sb, false);
+			return PullTextCarefully (textPool, sb, chars_to_pull, false);
 		}
 
-		private bool PullHotText (StringBuilder sb)
+		private bool PullHotText (StringBuilder sb, int chars_to_pull)
 		{
-			return PullTextCarefully (hotPool, sb, true);
+			return PullTextCarefully (hotPool, sb, -1, true);
 		}
 
 		public TextReader GetTextReader ()
 		{
-			PullingReader pr = new PullingReader (new PullingReader.Pull (PullText));
+			PullingReader pr = new PullingReader (
+				new PullingReader.Pull (PullText),
+				new PullingReader.DoClose (Close));
 			return pr;
 		}
 
@@ -754,46 +909,50 @@ namespace Beagle.Daemon {
 			return new PullingReader (new PullingReader.Pull (PullHotText));
 		}
 
-		public IEnumerable Properties {
-			get { return propertyPool; }
-		}
-
 		//////////////////////////////
+		//////////// Default implementation of generated indexables
 
-		// This is used primarily for the generation of URIs for the
-		// child indexables that can be created as a result of the
-		// filtering process.
+		private ArrayList generated_indexables = new ArrayList ();
 
-		private Uri uri = null;
-
-		public Uri Uri {
-			get { return uri; }
-			set { uri = value; }
+		public virtual bool HasGeneratedIndexable {
+			get { return generated_indexables.Count > 0; }
 		}
 
-		private Uri display_uri = null;
-
-		public Uri DisplayUri {
-			get { return display_uri; }
-			set { display_uri = value; }
-		}
-
-		//////////////////////////////
-
-		private ArrayList child_indexables = new ArrayList ();
-
-		protected void AddChildIndexable (Indexable indexable)
+		/// <summary>
+		/// Good filters should replace this by an IEnumerable that does not require generating
+		/// all the indexables beforehand
+		/// </summary>
+		/// <param name="indexable">
+		/// A <see cref="Indexable"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		public virtual bool GenerateNextIndexable (out Indexable indexable)
 		{
-			this.child_indexables.Add (indexable);
-		}
-		
-		protected void AddChildIndexables (ICollection indexables)
-		{
-			this.child_indexables.AddRange (indexables);
+			indexable = null;
+			if (generated_indexables.Count == 0)
+				return false;
+
+			indexable = (Indexable) generated_indexables [0];
+			generated_indexables.RemoveAt (0);
+			return true;
 		}
 
-		public ArrayList ChildIndexables {
-			get { return this.child_indexables; }
+		protected void AddIndexable (Indexable indexable)
+		{
+			this.generated_indexables.Add (indexable);
+		}
+
+		protected void AddIndexables (ICollection indexables)
+		{
+			this.generated_indexables.AddRange (indexables);
+		}
+
+		public void CleanGeneratedIndexables ()
+		{
+			foreach (Indexable indexable in generated_indexables)
+				indexable.Cleanup ();
 		}
 	}
 
