@@ -85,21 +85,19 @@ namespace Beagle.IndexHelper {
 			bool run_by_hand = (Environment.GetEnvironmentVariable ("BEAGLE_RUN_HELPER_BY_HAND") != null);
 			bool log_in_fg = (Environment.GetEnvironmentVariable ("BEAGLE_LOG_IN_THE_FOREGROUND_PLEASE") != null);
 
-			if (args.Length == 1 && args [0] == "--disable-text-cache")
-				disable_textcache = true;
-			else
-				disable_textcache = false;
-			
-			// FIXME: We always turn on full debugging output!  We are still
-			// debugging this code, after all...
-			//bool debug = (Environment.GetEnvironmentVariable ("BEAGLE_DEBUG_FLAG_IS_SET") != null);
+			bool debug = false, disable_textcache = false;
+
+			foreach (string arg in args)
+				if (arg == "--disable-text-cache")
+					disable_textcache = true;
+				else if (arg == "--debug")
+					debug = true;
 
 			last_activity = DateTime.Now;
 
 			Log.Initialize (PathFinder.LogDir,
 					"IndexHelper",
-					//debug ? LogLevel.Debug : LogLevel.Warn,
-					LogLevel.Debug,
+					debug ? LogLevel.Debug : LogLevel.Warn,
 					run_by_hand || log_in_fg);
 
 			Log.Always ("Starting Index Helper process (version {0})", ExternalStringsHack.Version);
