@@ -1,9 +1,10 @@
 /*
- * Copyright 2004 The Apache Software Foundation
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  * http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -15,6 +16,7 @@
  */
 
 using System;
+
 using Analyzer = Lucene.Net.Analysis.Analyzer;
 using BooleanClause = Lucene.Net.Search.BooleanClause;
 using BooleanQuery = Lucene.Net.Search.BooleanQuery;
@@ -30,12 +32,44 @@ namespace Lucene.Net.QueryParsers
 	/// </summary>
 	/// <author>  <a href="mailto:kelvin@relevanz.com">Kelvin Tan</a>, Daniel Naber
 	/// </author>
-	/// <version>  $Revision: 1.4 $
+	/// <version>  $Revision: 472959 $
 	/// </version>
 	public class MultiFieldQueryParser : QueryParser
 	{
 		
 		private System.String[] fields;
+		private System.Collections.IDictionary boosts;
+		
+		/// <summary> Creates a MultiFieldQueryParser. 
+		/// Allows passing of a map with term to Boost, and the boost to apply to each term.
+		/// 
+		/// <p>It will, when parse(String query)
+		/// is called, construct a query like this (assuming the query consists of
+		/// two terms and you specify the two fields <code>title</code> and <code>body</code>):</p>
+		/// 
+		/// <code>
+		/// (title:term1 body:term1) (title:term2 body:term2)
+		/// </code>
+		/// 
+		/// <p>When setDefaultOperator(AND_OPERATOR) is set, the result will be:</p>
+		/// 
+		/// <code>
+		/// +(title:term1 body:term1) +(title:term2 body:term2)
+		/// </code>
+		/// 
+		/// <p>When you pass a boost (title=>5 body=>10) you can get </p>
+		/// 
+		/// <code>
+		/// +(title:term1^5.0 body:term1^10.0) +(title:term2^5.0 body:term2^10.0)
+		/// </code>
+		/// 
+		/// <p>In other words, all the query's terms must appear, but it doesn't matter in
+		/// what fields they appear.</p>
+		/// </summary>
+		public MultiFieldQueryParser(System.String[] fields, Analyzer analyzer, System.Collections.IDictionary boosts):this(fields, analyzer)
+		{
+			this.boosts = boosts;
+		}
 		
 		/// <summary> Creates a MultiFieldQueryParser.
 		/// 
@@ -56,12 +90,12 @@ namespace Lucene.Net.QueryParsers
 		/// <p>In other words, all the query's terms must appear, but it doesn't matter in
 		/// what fields they appear.</p>
 		/// </summary>
-		public MultiFieldQueryParser(System.String[] fields, Analyzer analyzer) : base(null, analyzer)
+		public MultiFieldQueryParser(System.String[] fields, Analyzer analyzer):base(null, analyzer)
 		{
 			this.fields = fields;
 		}
 		
-		protected internal override Query GetFieldQuery(System.String field, System.String queryText, int slop)
+		public override Query GetFieldQuery(System.String field, System.String queryText, int slop)
 		{
 			if (field == null)
 			{
@@ -71,6 +105,17 @@ namespace Lucene.Net.QueryParsers
 					Query q = base.GetFieldQuery(fields[i], queryText);
 					if (q != null)
 					{
+						//If the user passes a map of boosts
+						if (boosts != null)
+						{
+							//Get the boost from the map and apply them
+							System.Single boost = (System.Single) boosts[fields[i]];
+							//if (boost != null)      // {{Aroush-2.1 there is no null for 'boost'
+                            if (boosts[fields[i]] != null)      // {{Aroush-2.1 will this line do?
+							{
+								q.SetBoost((float) boost);
+							}
+						}
 						if (q is PhraseQuery)
 						{
 							((PhraseQuery) q).SetSlop(slop);
@@ -91,26 +136,13 @@ namespace Lucene.Net.QueryParsers
 		}
 		
 		
-		protected internal override Query GetFieldQuery(System.String field, System.String queryText)
+		public override Query GetFieldQuery(System.String field, System.String queryText)
 		{
 			return GetFieldQuery(field, queryText, 0);
 		}
 		
-		/// <deprecated> use {@link #GetFieldQuery(String, String)}
-		/// </deprecated>
-		protected internal override Query GetFieldQuery(System.String field, Analyzer analyzer, System.String queryText)
-		{
-			return GetFieldQuery(field, queryText);
-		}
 		
-		/// <deprecated> use {@link #GetFuzzyQuery(String, String, float)}
-		/// </deprecated>
-		protected internal override Query GetFuzzyQuery(System.String field, System.String termStr)
-		{
-			return GetFuzzyQuery(field, termStr, fuzzyMinSim);
-		}
-		
-		protected internal override Query GetFuzzyQuery(System.String field, System.String termStr, float minSimilarity)
+		public override Query GetFuzzyQuery(System.String field, System.String termStr, float minSimilarity)
 		{
 			if (field == null)
 			{
@@ -124,7 +156,7 @@ namespace Lucene.Net.QueryParsers
 			return base.GetFuzzyQuery(field, termStr, minSimilarity);
 		}
 		
-		protected internal override Query GetPrefixQuery(System.String field, System.String termStr)
+		public override Query GetPrefixQuery(System.String field, System.String termStr)
 		{
 			if (field == null)
 			{
@@ -138,7 +170,7 @@ namespace Lucene.Net.QueryParsers
 			return base.GetPrefixQuery(field, termStr);
 		}
 		
-		protected internal override Query GetWildcardQuery(System.String field, System.String termStr)
+		public override Query GetWildcardQuery(System.String field, System.String termStr)
 		{
 			if (field == null)
 			{
@@ -152,15 +184,8 @@ namespace Lucene.Net.QueryParsers
 			return base.GetWildcardQuery(field, termStr);
 		}
 		
-		/// <throws>  ParseException </throws>
-		/// <deprecated> use {@link #GetRangeQuery(String, String, String, boolean)}
-		/// </deprecated>
-		protected internal override Query GetRangeQuery(System.String field, Analyzer analyzer, System.String part1, System.String part2, bool inclusive)
-		{
-			return GetRangeQuery(field, part1, part2, inclusive);
-		}
 		
-		protected internal override Query GetRangeQuery(System.String field, System.String part1, System.String part2, bool inclusive)
+		public override Query GetRangeQuery(System.String field, System.String part1, System.String part2, bool inclusive)
 		{
 			if (field == null)
 			{
@@ -175,66 +200,7 @@ namespace Lucene.Net.QueryParsers
 		}
 		
 		
-		/// <deprecated> 
-		/// </deprecated>
-		public const int NORMAL_FIELD = 0;
-		/// <deprecated> 
-		/// </deprecated>
-		public const int REQUIRED_FIELD = 1;
-		/// <deprecated> 
-		/// </deprecated>
-		public const int PROHIBITED_FIELD = 2;
 		
-		/// <deprecated> use {@link #MultiFieldQueryParser(String[], Analyzer)} instead
-		/// </deprecated>
-		public MultiFieldQueryParser(QueryParserTokenManager tm):base(tm)
-		{
-		}
-		
-		/// <deprecated> use {@link #MultiFieldQueryParser(String[], Analyzer)} instead
-		/// </deprecated>
-		public MultiFieldQueryParser(CharStream stream):base(stream)
-		{
-		}
-		
-		/// <deprecated> use {@link #MultiFieldQueryParser(String[], Analyzer)} instead
-		/// </deprecated>
-		public MultiFieldQueryParser(System.String f, Analyzer a):base(f, a)
-		{
-		}
-		
-		/// <summary> Parses a query which searches on the fields specified.
-		/// If x fields are specified, this effectively constructs:
-		/// 
-		/// <code>
-		/// (field1:query) (field2:query) (field3:query)...(fieldx:query)
-		/// </code>
-		/// 
-		/// </summary>
-		/// <param name="query">Query string to parse
-		/// </param>
-		/// <param name="fields">Fields to search on
-		/// </param>
-		/// <param name="analyzer">Analyzer to use
-		/// </param>
-		/// <throws>  ParseException if query parsing fails </throws>
-		/// <throws>  TokenMgrError if query parsing fails </throws>
-		/// <deprecated> use {@link #Parse(String)} instead but note that it
-		/// returns a different query for queries where all terms are required:
-		/// its query excepts all terms, no matter in what field they occur whereas
-		/// the query built by this (deprecated) method expected all terms in all fields 
-		/// at the same time.
-		/// </deprecated>
-		public static Query Parse(System.String query, System.String[] fields, Analyzer analyzer)
-		{
-			BooleanQuery bQuery = new BooleanQuery();
-			for (int i = 0; i < fields.Length; i++)
-			{
-				Query q = Parse(query, fields[i], analyzer);
-				bQuery.Add(q, BooleanClause.Occur.SHOULD);
-			}
-			return bQuery;
-		}
 		
 		/// <summary> Parses a query which searches on the fields specified.
 		/// <p>
@@ -252,7 +218,6 @@ namespace Lucene.Net.QueryParsers
 		/// <param name="analyzer">Analyzer to use
 		/// </param>
 		/// <throws>  ParseException if query parsing fails </throws>
-		/// <throws>  TokenMgrError if query parsing fails </throws>
 		/// <throws>  IllegalArgumentException if the length of the queries array differs </throws>
 		/// <summary>  from the length of the fields array
 		/// </summary>
@@ -270,72 +235,6 @@ namespace Lucene.Net.QueryParsers
 			return bQuery;
 		}
 		
-		/// <summary> Parses a query, searching on the fields specified.
-		/// Use this if you need to specify certain fields as required,
-		/// and others as prohibited.
-		/// <p><pre>
-		/// Usage:
-		/// <code>
-		/// String[] fields = {"filename", "contents", "description"};
-		/// int[] flags = {MultiFieldQueryParser.NORMAL_FIELD,
-		/// MultiFieldQueryParser.REQUIRED_FIELD,
-		/// MultiFieldQueryParser.PROHIBITED_FIELD,};
-		/// parse(query, fields, flags, analyzer);
-		/// </code>
-		/// </pre>
-		/// <p>
-		/// The code above would construct a query:
-		/// <pre>
-		/// <code>
-		/// (filename:query) +(contents:query) -(description:query)
-		/// </code>
-		/// </pre>
-		/// 
-		/// </summary>
-		/// <param name="query">Query string to parse
-		/// </param>
-		/// <param name="fields">Fields to search on
-		/// </param>
-		/// <param name="flags">Flags describing the fields
-		/// </param>
-		/// <param name="analyzer">Analyzer to use
-		/// </param>
-		/// <throws>  ParseException if query parsing fails </throws>
-		/// <throws>  TokenMgrError if query parsing fails </throws>
-		/// <throws>  IllegalArgumentException if the length of the fields array differs </throws>
-		/// <summary>  from the length of the flags array
-		/// </summary>
-		/// <deprecated> use {@link #Parse(String, String[], BooleanClause.Occur[], Analyzer)} instead
-		/// </deprecated>
-		public static Query Parse(System.String query, System.String[] fields, int[] flags, Analyzer analyzer)
-		{
-			if (fields.Length != flags.Length)
-				throw new System.ArgumentException("fields.length != flags.length");
-			BooleanQuery bQuery = new BooleanQuery();
-			for (int i = 0; i < fields.Length; i++)
-			{
-				QueryParser qp = new QueryParser(fields[i], analyzer);
-				Query q = qp.Parse(query);
-				int flag = flags[i];
-				switch (flag)
-				{
-					
-					case REQUIRED_FIELD: 
-						bQuery.Add(q, BooleanClause.Occur.MUST);
-						break;
-					
-					case PROHIBITED_FIELD: 
-						bQuery.Add(q, BooleanClause.Occur.MUST_NOT);
-						break;
-					
-					default: 
-						bQuery.Add(q, BooleanClause.Occur.SHOULD);
-						break;
-					
-				}
-			}
-			return bQuery;
-		}
 		
 		/// <summary> Parses a query, searching on the fields specified.
 		/// Use this if you need to specify certain fields as required,
@@ -368,7 +267,6 @@ namespace Lucene.Net.QueryParsers
 		/// <param name="analyzer">Analyzer to use
 		/// </param>
 		/// <throws>  ParseException if query parsing fails </throws>
-		/// <throws>  TokenMgrError if query parsing fails </throws>
 		/// <throws>  IllegalArgumentException if the length of the fields array differs </throws>
 		/// <summary>  from the length of the flags array
 		/// </summary>
@@ -386,71 +284,6 @@ namespace Lucene.Net.QueryParsers
 			return bQuery;
 		}
 		
-		/// <summary> Parses a query, searching on the fields specified. Use this if you need to
-		/// specify certain fields as required, and others as prohibited.
-		/// <p>
-		/// <pre>
-		/// Usage:
-		/// <code>
-		/// String[] fields = { &quot;filename&quot;, &quot;contents&quot;, &quot;description&quot; };
-		/// int[] flags = { MultiFieldQueryParser.NORMAL_FIELD,
-		/// MultiFieldQueryParser.REQUIRED_FIELD,
-		/// MultiFieldQueryParser.PROHIBITED_FIELD, };
-		/// parse(query, fields, flags, analyzer);
-		/// </code>
-		/// </pre>
-		/// 
-		/// <p>
-		/// The code above would construct a query:
-		/// <pre>
-		/// <code>
-		/// (filename:query1) +(contents:query2) -(description:query3)
-		/// </code>
-		/// </pre>
-		/// 
-		/// </summary>
-		/// <param name="queries">Queries string to parse
-		/// </param>
-		/// <param name="fields">Fields to search on
-		/// </param>
-		/// <param name="flags">Flags describing the fields
-		/// </param>
-		/// <param name="analyzer">Analyzer to use
-		/// </param>
-		/// <throws>  ParseException if query parsing fails </throws>
-		/// <throws>  TokenMgrError if query parsing fails </throws>
-		/// <throws>  IllegalArgumentException if the length of the queries, fields, and flags array differ </throws>
-		/// <deprecated> use {@link #Parse(String[], String[], BooleanClause.Occur[], Analyzer)} instead
-		/// </deprecated>
-		public static Query Parse(System.String[] queries, System.String[] fields, int[] flags, Analyzer analyzer)
-		{
-			if (!(queries.Length == fields.Length && queries.Length == flags.Length))
-				throw new System.ArgumentException("queries, fields, and flags array have have different length");
-			BooleanQuery bQuery = new BooleanQuery();
-			for (int i = 0; i < fields.Length; i++)
-			{
-				QueryParser qp = new QueryParser(fields[i], analyzer);
-				Query q = qp.Parse(queries[i]);
-				int flag = flags[i];
-				switch (flag)
-				{
-					
-					case REQUIRED_FIELD: 
-						bQuery.Add(q, BooleanClause.Occur.MUST);
-						break;
-					
-					case PROHIBITED_FIELD: 
-						bQuery.Add(q, BooleanClause.Occur.MUST_NOT);
-						break;
-					
-					default: 
-						bQuery.Add(q, BooleanClause.Occur.SHOULD);
-						break;
-					
-				}
-			}
-			return bQuery;
-		}
 		
 		/// <summary> Parses a query, searching on the fields specified.
 		/// Use this if you need to specify certain fields as required,
@@ -484,7 +317,6 @@ namespace Lucene.Net.QueryParsers
 		/// <param name="analyzer">Analyzer to use
 		/// </param>
 		/// <throws>  ParseException if query parsing fails </throws>
-		/// <throws>  TokenMgrError if query parsing fails </throws>
 		/// <throws>  IllegalArgumentException if the length of the queries, fields, </throws>
 		/// <summary>  and flags array differ
 		/// </summary>
