@@ -1,9 +1,10 @@
 /*
- * Copyright 2004 The Apache Software Foundation
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  * http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -37,6 +38,21 @@ namespace Lucene.Net.Search
 			this.description = description;
 		}
 		
+		/// <summary> Indicates wether or not this Explanation models a good match.
+		/// 
+		/// <p>
+		/// By default, an Explanation represents a "match" if the value is positive.
+		/// </p>
+		/// </summary>
+		/// <seealso cref="#getValue">
+		/// </seealso>
+		public virtual bool IsMatch()
+		{
+			return (0.0f < GetValue());
+		}
+		
+		
+		
 		/// <summary>The value assigned to this explanation node. </summary>
 		public virtual float GetValue()
 		{
@@ -59,6 +75,14 @@ namespace Lucene.Net.Search
 			this.description = description;
 		}
 		
+		/// <summary> A short one line summary which should contain all high level
+		/// information about this Explanation, without the "Details"
+		/// </summary>
+		protected internal virtual System.String GetSummary()
+		{
+			return GetValue() + " = " + GetDescription();
+		}
+		
 		/// <summary>The sub-nodes of this explanation node. </summary>
 		public virtual Explanation[] GetDetails()
 		{
@@ -80,16 +104,14 @@ namespace Lucene.Net.Search
 		{
 			return ToString(0);
 		}
-		private System.String ToString(int depth)
+		protected internal virtual System.String ToString(int depth)
 		{
 			System.Text.StringBuilder buffer = new System.Text.StringBuilder();
 			for (int i = 0; i < depth; i++)
 			{
 				buffer.Append("  ");
 			}
-			buffer.Append(GetValue());
-			buffer.Append(" = ");
-			buffer.Append(GetDescription());
+			buffer.Append(GetSummary());
 			buffer.Append("\n");
 			
 			Explanation[] details = GetDetails();
@@ -112,10 +134,8 @@ namespace Lucene.Net.Search
 			buffer.Append("<ul>\n");
 			
 			buffer.Append("<li>");
-			buffer.Append(GetValue());
-			buffer.Append(" = ");
-			buffer.Append(GetDescription());
-			buffer.Append("</li>\n");
+			buffer.Append(GetSummary());
+			buffer.Append("<br />\n");
 			
 			Explanation[] details = GetDetails();
 			if (details != null)
@@ -126,6 +146,7 @@ namespace Lucene.Net.Search
 				}
 			}
 			
+			buffer.Append("</li>\n");
 			buffer.Append("</ul>\n");
 			
 			return buffer.ToString();
