@@ -1,9 +1,10 @@
 /*
- * Copyright 2004 The Apache Software Foundation
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  * http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -15,7 +16,7 @@
  */
 
 using System;
-using Field = Lucene.Net.Documents.Field;
+
 using IndexReader = Lucene.Net.Index.IndexReader;
 using IndexWriter = Lucene.Net.Index.IndexWriter;
 using Term = Lucene.Net.Index.Term;
@@ -27,61 +28,265 @@ namespace Lucene.Net.Search
 	/// <summary>Expert: Scoring API.
 	/// <p>Subclasses implement search scoring.
 	/// 
-	/// <p>The score of query <code>q</code> for document <code>d</code> is defined
-	/// in terms of these methods as follows:
+	/// <p>The score of query <code>q</code> for document <code>d</code> correlates to the
+	/// cosine-distance or dot-product between document and query vectors in a
+	/// <a href="http://en.wikipedia.org/wiki/Vector_Space_Model">
+	/// Vector Space Model (VSM) of Information Retrieval</a>.
+	/// A document whose vector is closer to the query vector in that model is scored higher.
 	/// 
-	/// <table cellpadding="0" cellspacing="0" border="0">
+	/// The score is computed as follows:
+	/// 
+	/// <P>
+	/// <table cellpadding="1" cellspacing="0" border="1" align="center">
+	/// <tr><td>
+	/// <table cellpadding="1" cellspacing="0" border="0" align="center">
 	/// <tr>
-	/// <td valign="middle" align="right" rowspan="2">score(q,d) =<br></td>
-	/// <td valign="middle" align="center">
-	/// <big><big><big><big><big>&Sigma;</big></big></big></big></big></td>
-	/// <td valign="middle"><small>
-	/// ( {@link #Tf(int) tf}(t in d) *
-	/// {@link #Idf(Term,Searcher) idf}(t)^2 *
-	/// {@link Query#getBoost getBoost}(t in q) *
-	/// {@link Field#getBoost getBoost}(t.field in d) *
-	/// {@link #LengthNorm(String,int) lengthNorm}(t.field in d) )
-	/// </small></td>
-	/// <td valign="middle" rowspan="2">&nbsp;*
-	/// {@link #Coord(int,int) coord}(q,d) *
-	/// {@link #QueryNorm(float) queryNorm}(sumOfSqaredWeights)
+	/// <td valign="middle" align="right" rowspan="1">
+	/// score(q,d) &nbsp; = &nbsp;
+	/// <A HREF="#formula_coord">coord(q,d)</A> &nbsp;&middot;&nbsp;
+	/// <A HREF="#formula_queryNorm">queryNorm(q)</A> &nbsp;&middot;&nbsp;
+	/// </td>
+	/// <td valign="bottom" align="center" rowspan="1">
+	/// <big><big><big>&sum;</big></big></big>
+	/// </td>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// <big><big>(</big></big>
+	/// <A HREF="#formula_tf">tf(t in d)</A> &nbsp;&middot;&nbsp;
+	/// <A HREF="#formula_idf">idf(t)</A><sup>2</sup> &nbsp;&middot;&nbsp;
+	/// <A HREF="#formula_termBoost">t.getBoost()</A>&nbsp;&middot;&nbsp;
+	/// <A HREF="#formula_norm">norm(t,d)</A>
+	/// <big><big>)</big></big>
 	/// </td>
 	/// </tr>
-	/// <tr>
-	/// <td valign="top" align="right">
-	/// <small>t in q</small>
-	/// </td>
+	/// <tr valigh="top">
+	/// <td></td>
+	/// <td align="center"><small>t in q</small></td>
+	/// <td></td>
 	/// </tr>
 	/// </table>
+	/// </td></tr>
+	/// </table>
 	/// 
 	/// <p> where
+	/// <ol>
+	/// <li>
+	/// <A NAME="formula_tf"></A>
+	/// <b>tf(t in d)</b>
+	/// correlates to the term's <i>frequency</i>,
+	/// defined as the number of times term <i>t</i> appears in the currently scored document <i>d</i>.
+	/// Documents that have more occurrences of a given term receive a higher score.
+	/// The default computation for <i>tf(t in d)</i> in
+	/// {@link Lucene.Net.Search.DefaultSimilarity#Tf(float) DefaultSimilarity} is:
 	/// 
-	/// <table cellpadding="0" cellspacing="0" border="0">
+	/// <br>&nbsp;<br>
+	/// <table cellpadding="2" cellspacing="2" border="0" align="center">
 	/// <tr>
-	/// <td valign="middle" align="right" rowspan="2">sumOfSqaredWeights =<br></td>
-	/// <td valign="middle" align="center">
-	/// <big><big><big><big><big>&Sigma;</big></big></big></big></big></td>
-	/// <td valign="middle"><small>
-	/// ( {@link #Idf(Term,Searcher) idf}(t) *
-	/// {@link Query#getBoost getBoost}(t in q) )^2
-	/// </small></td>
-	/// </tr>
-	/// <tr>
-	/// <td valign="top" align="right">
-	/// <small>t in q</small>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// {@link Lucene.Net.Search.DefaultSimilarity#Tf(float) tf(t in d)} &nbsp; = &nbsp;
+	/// </td>
+	/// <td valign="top" align="center" rowspan="1">
+	/// frequency<sup><big>&frac12;</big></sup>
 	/// </td>
 	/// </tr>
 	/// </table>
+	/// <br>&nbsp;<br>
+	/// </li>
 	/// 
-	/// <p> Note that the above formula is motivated by the cosine-distance or dot-product
-	/// between document and query vector, which is implemented by {@link DefaultSimilarity}.
+	/// <li>
+	/// <A NAME="formula_idf"></A>
+	/// <b>idf(t)</b> stands for Inverse Document Frequency. This value
+	/// correlates to the inverse of <i>docFreq</i>
+	/// (the number of documents in which the term <i>t</i> appears).
+	/// This means rarer terms give higher contribution to the total score.
+	/// The default computation for <i>idf(t)</i> in
+	/// {@link Lucene.Net.Search.DefaultSimilarity#Idf(int, int) DefaultSimilarity} is:
+	/// 
+	/// <br>&nbsp;<br>
+	/// <table cellpadding="2" cellspacing="2" border="0" align="center">
+	/// <tr>
+	/// <td valign="middle" align="right">
+	/// {@link Lucene.Net.Search.DefaultSimilarity#Idf(int, int) idf(t)}&nbsp; = &nbsp;
+	/// </td>
+	/// <td valign="middle" align="center">
+	/// 1 + log <big>(</big>
+	/// </td>
+	/// <td valign="middle" align="center">
+	/// <table>
+	/// <tr><td align="center"><small>numDocs</small></td></tr>
+	/// <tr><td align="center">&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;</td></tr>
+	/// <tr><td align="center"><small>docFreq+1</small></td></tr>
+	/// </table>
+	/// </td>
+	/// <td valign="middle" align="center">
+	/// <big>)</big>
+	/// </td>
+	/// </tr>
+	/// </table>
+	/// <br>&nbsp;<br>
+	/// </li>
+	/// 
+	/// <li>
+	/// <A NAME="formula_coord"></A>
+	/// <b>coord(q,d)</b>
+	/// is a score factor based on how many of the query terms are found in the specified document.
+	/// Typically, a document that contains more of the query's terms will receive a higher score
+	/// than another document with fewer query terms.
+	/// This is a search time factor computed in
+	/// {@link #Coord(int, int) coord(q,d)}
+	/// by the Similarity in effect at search time.
+	/// <br>&nbsp;<br>
+	/// </li>
+	/// 
+	/// <li><b>
+	/// <A NAME="formula_queryNorm"></A>
+	/// queryNorm(q)
+	/// </b>
+	/// is a normalizing factor used to make scores between queries comparable.
+	/// This factor does not affect document ranking (since all ranked documents are multiplied by the same factor),
+	/// but rather just attempts to make scores from different queries (or even different indexes) comparable.
+	/// This is a search time factor computed by the Similarity in effect at search time.
+	/// 
+	/// The default computation in
+	/// {@link Lucene.Net.Search.DefaultSimilarity#QueryNorm(float) DefaultSimilarity}
+	/// is:
+	/// <br>&nbsp;<br>
+	/// <table cellpadding="1" cellspacing="0" border="0" align="center">
+	/// <tr>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// queryNorm(q)  &nbsp; = &nbsp;
+	/// {@link Lucene.Net.Search.DefaultSimilarity#QueryNorm(float) queryNorm(sumOfSquaredWeights)}
+	/// &nbsp; = &nbsp;
+	/// </td>
+	/// <td valign="middle" align="center" rowspan="1">
+	/// <table>
+	/// <tr><td align="center"><big>1</big></td></tr>
+	/// <tr><td align="center"><big>
+	/// &ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;&ndash;
+	/// </big></td></tr>
+	/// <tr><td align="center">sumOfSquaredWeights<sup><big>&frac12;</big></sup></td></tr>
+	/// </table>
+	/// </td>
+	/// </tr>
+	/// </table>
+	/// <br>&nbsp;<br>
+	/// 
+	/// The sum of squared weights (of the query terms) is
+	/// computed by the query {@link Lucene.Net.Search.Weight} object.
+	/// For example, a {@link Lucene.Net.Search.BooleanQuery boolean query}
+	/// computes this value as:
+	/// 
+	/// <br>&nbsp;<br>
+	/// <table cellpadding="1" cellspacing="0" border="0"n align="center">
+	/// <tr>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// {@link Lucene.Net.Search.Weight#SumOfSquaredWeights() sumOfSquaredWeights} &nbsp; = &nbsp;
+	/// {@link Lucene.Net.Search.Query#GetBoost() q.getBoost()} <sup><big>2</big></sup>
+	/// &nbsp;&middot;&nbsp;
+	/// </td>
+	/// <td valign="bottom" align="center" rowspan="1">
+	/// <big><big><big>&sum;</big></big></big>
+	/// </td>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// <big><big>(</big></big>
+	/// <A HREF="#formula_idf">idf(t)</A> &nbsp;&middot;&nbsp;
+	/// <A HREF="#formula_termBoost">t.getBoost()</A>
+	/// <big><big>) <sup>2</sup> </big></big>
+	/// </td>
+	/// </tr>
+	/// <tr valigh="top">
+	/// <td></td>
+	/// <td align="center"><small>t in q</small></td>
+	/// <td></td>
+	/// </tr>
+	/// </table>
+	/// <br>&nbsp;<br>
+	/// 
+	/// </li>
+	/// 
+	/// <li>
+	/// <A NAME="formula_termBoost"></A>
+	/// <b>t.getBoost()</b>
+	/// is a search time boost of term <i>t</i> in the query <i>q</i> as
+	/// specified in the query text
+	/// (see <A HREF="../../../../../queryparsersyntax.html#Boosting a Term">query syntax</A>),
+	/// or as set by application calls to
+	/// {@link Lucene.Net.Search.Query#SetBoost(float) setBoost()}.
+	/// Notice that there is really no direct API for accessing a boost of one term in a multi term query,
+	/// but rather multi terms are represented in a query as multi
+	/// {@link Lucene.Net.Search.TermQuery TermQuery} objects,
+	/// and so the boost of a term in the query is accessible by calling the sub-query
+	/// {@link Lucene.Net.Search.Query#GetBoost() getBoost()}.
+	/// <br>&nbsp;<br>
+	/// </li>
+	/// 
+	/// <li>
+	/// <A NAME="formula_norm"></A>
+	/// <b>norm(t,d)</b> encapsulates a few (indexing time) boost and length factors:
+	/// 
+	/// <ul>
+	/// <li><b>Document boost</b> - set by calling
+	/// {@link Lucene.Net.Documents.Document#SetBoost(float) doc.setBoost()}
+	/// before adding the document to the index.
+	/// </li>
+	/// <li><b>Field boost</b> - set by calling
+	/// {@link Lucene.Net.Documents.Fieldable#SetBoost(float) field.setBoost()}
+	/// before adding the field to a document.
+	/// </li>
+	/// <li>{@link #LengthNorm(String, int) <b>lengthNorm</b>(field)} - computed
+	/// when the document is added to the index in accordance with the number of tokens
+	/// of this field in the document, so that shorter fields contribute more to the score.
+	/// LengthNorm is computed by the Similarity class in effect at indexing.
+	/// </li>
+	/// </ul>
+	/// 
+	/// <p>
+	/// When a document is added to the index, all the above factors are multiplied.
+	/// If the document has multiple fields with the same name, all their boosts are multiplied together:
+	/// 
+	/// <br>&nbsp;<br>
+	/// <table cellpadding="1" cellspacing="0" border="0"n align="center">
+	/// <tr>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// norm(t,d) &nbsp; = &nbsp;
+	/// {@link Lucene.Net.Documents.Document#GetBoost() doc.getBoost()}
+	/// &nbsp;&middot;&nbsp;
+	/// {@link #LengthNorm(String, int) lengthNorm(field)}
+	/// &nbsp;&middot;&nbsp;
+	/// </td>
+	/// <td valign="bottom" align="center" rowspan="1">
+	/// <big><big><big>&prod;</big></big></big>
+	/// </td>
+	/// <td valign="middle" align="right" rowspan="1">
+	/// {@link Lucene.Net.Documents.Fieldable#GetBoost() f.getBoost}()
+	/// </td>
+	/// </tr>
+	/// <tr valigh="top">
+	/// <td></td>
+	/// <td align="center"><small>field <i><b>f</b></i> in <i>d</i> named as <i><b>t</b></i></small></td>
+	/// <td></td>
+	/// </tr>
+	/// </table>
+	/// <br>&nbsp;<br>
+	/// However the resulted <i>norm</i> value is {@link #EncodeNorm(float) encoded} as a single byte
+	/// before being stored.
+	/// At search time, the norm byte value is read from the index
+	/// {@link Lucene.Net.Store.Directory directory} and
+	/// {@link #DecodeNorm(byte) decoded} back to a float <i>norm</i> value.
+	/// This encoding/decoding, while reducing index size, comes with the price of
+	/// precision loss - it is not guaranteed that decode(encode(x)) = x.
+	/// For instance, decode(encode(0.89)) = 0.75.
+	/// Also notice that search time is too late to modify this <i>norm</i> part of scoring, e.g. by
+	/// using a different {@link Similarity} for search.
+	/// <br>&nbsp;<br>
+	/// </li>
+	/// </ol>
 	/// 
 	/// </summary>
-	/// <seealso cref="SetDefault(Similarity)">
+	/// <seealso cref="#SetDefault(Similarity)">
 	/// </seealso>
-	/// <seealso cref="IndexWriter.SetSimilarity(Similarity)">
+	/// <seealso cref="IndexWriter#SetSimilarity(Similarity)">
 	/// </seealso>
-	/// <seealso cref="Searcher.SetSimilarity(Similarity)">
+	/// <seealso cref="Searcher#SetSimilarity(Similarity)">
 	/// </seealso>
 	[Serializable]
 	public abstract class Similarity
@@ -93,9 +298,9 @@ namespace Lucene.Net.Search
 		/// code.
 		/// 
 		/// </summary>
-		/// <seealso cref="Searcher.SetSimilarity(Similarity)">
+		/// <seealso cref="Searcher#SetSimilarity(Similarity)">
 		/// </seealso>
-		/// <seealso cref="IndexWriter.SetSimilarity(Similarity)">
+		/// <seealso cref="IndexWriter#SetSimilarity(Similarity)">
 		/// </seealso>
 		public static void  SetDefault(Similarity similarity)
 		{
@@ -108,9 +313,9 @@ namespace Lucene.Net.Search
 		/// <p>This is initially an instance of {@link DefaultSimilarity}.
 		/// 
 		/// </summary>
-		/// <seealso cref="Searcher.SetSimilarity(Similarity)">
+		/// <seealso cref="Searcher#SetSimilarity(Similarity)">
 		/// </seealso>
-		/// <seealso cref="IndexWriter.SetSimilarity(Similarity)">
+		/// <seealso cref="IndexWriter#SetSimilarity(Similarity)">
 		/// </seealso>
 		public static Similarity GetDefault()
 		{
@@ -121,7 +326,7 @@ namespace Lucene.Net.Search
 		private static readonly float[] NORM_TABLE = new float[256];
 		
 		/// <summary>Decodes a normalization factor stored in an index.</summary>
-		/// <seealso cref="EncodeNorm(float)">
+		/// <seealso cref="#EncodeNorm(float)">
 		/// </seealso>
 		public static float DecodeNorm(byte b)
 		{
@@ -129,7 +334,7 @@ namespace Lucene.Net.Search
 		}
 		
 		/// <summary>Returns a table for decoding normalization bytes.</summary>
-		/// <seealso cref="EncodeNorm(float)">
+		/// <seealso cref="#EncodeNorm(float)">
 		/// </seealso>
 		public static float[] GetNormDecoder()
 		{
@@ -146,7 +351,7 @@ namespace Lucene.Net.Search
 		/// and larger values when <code>numTokens</code> is small.
 		/// 
 		/// <p>That these values are computed under {@link
-		/// IndexWriter#AddDocument(Lucene.Net.document.Document)} and stored then using
+		/// IndexWriter#AddDocument(Lucene.Net.Documents.Document)} and stored then using
 		/// {@link #EncodeNorm(float)}.  Thus they have limited precision, and documents
 		/// must be re-indexed if this method is altered.
 		/// 
@@ -159,7 +364,7 @@ namespace Lucene.Net.Search
 		/// <returns> a normalization factor for hits on this field of this document
 		/// 
 		/// </returns>
-		/// <seealso cref="Field.SetBoost(float)">
+		/// <seealso cref="Lucene.Net.Documents.Field#SetBoost(float)">
 		/// </seealso>
 		public abstract float LengthNorm(System.String fieldName, int numTokens);
 		
@@ -189,7 +394,7 @@ namespace Lucene.Net.Search
 		/// value.
 		/// 
 		/// </summary>
-		/// <seealso cref="Field.SetBoost(float)">
+		/// <seealso cref="Lucene.Net.Documents.Field#SetBoost(float)">
 		/// </seealso>
 		/// <seealso cref="SmallFloat">
 		/// </seealso>
@@ -231,7 +436,7 @@ namespace Lucene.Net.Search
 		/// when it is large.
 		/// 
 		/// </summary>
-		/// <seealso cref="PhraseQuery.SetSlop(int)">
+		/// <seealso cref="PhraseQuery#SetSlop(int)">
 		/// </seealso>
 		/// <param name="distance">the edit distance of this sloppy phrase match
 		/// </param>
@@ -276,7 +481,7 @@ namespace Lucene.Net.Search
 		/// </returns>
 		public virtual float Idf(Term term, Searcher searcher)
 		{
-			return Ldf(searcher.DocFreq(term), searcher.MaxDoc());
+			return Idf(searcher.DocFreq(term), searcher.MaxDoc());
 		}
 		
 		/// <summary>Computes a score factor for a phrase.
@@ -318,7 +523,7 @@ namespace Lucene.Net.Search
 		/// </param>
 		/// <returns> a score factor based on the term's document frequency
 		/// </returns>
-		public abstract float Ldf(int docFreq, int numDocs);
+		public abstract float Idf(int docFreq, int numDocs);
 		
 		/// <summary>Computes a score factor based on the fraction of all query terms that a
 		/// document contains.  This value is multiplied into scores.
