@@ -1,7 +1,7 @@
 //
 // QueryDriver.cs
 //
-// Copyright (C) 2004 Novell, Inc.
+// Copyright (C) 2004-2007 Novell, Inc.
 //
 
 //
@@ -27,13 +27,16 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 using System.Reflection;
 using System.Text;
 using System.Threading;
 using Beagle.Util;
+using System.Xml.Serialization;
+
 namespace Beagle.Daemon {
 	
-	public class QueryDriver {
+	public static class QueryDriver {
 
 		// Contains list of queryables explicitly asked by --allow-backend or --backend name
 		// --allow-backend/--backend name : dont read config information and only start backend 'name'
@@ -54,15 +57,18 @@ namespace Beagle.Daemon {
 			// set flag here to stop Allow() from calling ReadBackendsFromConf() again
 			done_reading_conf = true;
 
+			Config config = Conf.Get (Conf.Names.DaemonConfig);
+
 			// To allow static indexes, "static" should be in allowed_queryables
-			if (Conf.Daemon.AllowStaticBackend)
+			if (config.GetOption (Conf.Names.AllowStaticBackend, false))
 				Allow ("static");
 
-			if (Conf.Daemon.DeniedBackends == null)
+			List<string[]> values = config.GetListOptionValues (Conf.Names.DeniedBackends);
+			if (values == null)
 				return;
 			
-			foreach (string name in Conf.Daemon.DeniedBackends)
-				denied_queryables.Add (name.ToLower ());
+			foreach (string[] name in values)
+				denied_queryables.Add (name [0].ToLower ());
 		}
 
 		static public void OnlyAllow (string name)
@@ -124,6 +130,7 @@ namespace Beagle.Daemon {
 
 		public static int IndexingDelay {
 			set { indexing_delay = value; }
+			get { return indexing_delay; }
 		}
 
 		//////////////////////////////////////////////////////////////////////////////////////
@@ -131,22 +138,15 @@ namespace Beagle.Daemon {
 		// Use introspection to find all classes that implement IQueryable, the construct
 		// associated Queryables objects.
 
-		static ArrayList queryables = new ArrayList ();
+		// To protect the list of iqueryables, since now we support adding/removing of
+		// static queryables at runtime
+		static ReaderWriterLock iqueryable_lock = new ReaderWriterLock ();
+
 		static Hashtable iqueryable_to_queryable = new Hashtable ();
-
-		static bool ThisApiSoVeryIsBroken (Type m, object criteria)
-		{
-			return m == (Type) criteria;
+		static ICollection Queryables {
+			get { return iqueryable_to_queryable.Values; }
 		}
 
-		static bool TypeImplementsInterface (Type t, Type iface)
-		{
-			Type[] impls = t.FindInterfaces (new TypeFilter (ThisApiSoVeryIsBroken),
-							 iface);
-			return impls.Length > 0;
-		}
-
-		// Find the types in the assembly that
 		// (1) register themselves in AssemblyInfo.cs:IQueryableTypes and
 		// (2) has a QueryableFlavor attribute attached
 		// assemble a Queryable object and stick it into our list of queryables.
@@ -155,7 +155,6 @@ namespace Beagle.Daemon {
 			int count = 0;
 
 			foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (IQueryableTypesAttribute))) {
-				bool type_accepted = false;
 				foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor))) {
 					if (! UseQueryable (flavor.Name))
 						continue;
@@ -179,62 +178,17 @@ namespace Beagle.Daemon {
 
 					if (iq != null) {
 						Queryable q = new Queryable (flavor, iq);
-						queryables.Add (q);
 						iqueryable_to_queryable [iq] = q;
 						++count;
-						type_accepted = true;
 						break;
 					}
 				}
-
-				if (! type_accepted)
-					continue;
-
-				object[] attributes = type.GetCustomAttributes (false);
-				foreach (object attribute in attributes) {
-					PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
-					if (mapping == null)
-						continue;
-					//Logger.Log.Debug (mapping.Keyword + " => " 
-					//		+ mapping.PropertyName + 
-					//		+ " is-keyword=" + mapping.IsKeyword + " (" 
-					//		+ mapping.Description + ") "
-					//		+ "(" + type.FullName + ")");
-					PropertyKeywordFu.RegisterMapping (mapping);
-				}
-					
 			}
 			Logger.Log.Debug ("Found {0} backends in {1}", count, assembly.Location);
 		}
 
 		////////////////////////////////////////////////////////
 
-		public static void ReadKeywordMappings ()
-		{
-			Logger.Log.Debug ("Reading mapping from filters");
-			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_FILTER_PATH", PathFinder.FilterDir);
-
-			foreach (Assembly assembly in assemblies) {
-				foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (FilterTypesAttribute))) {
-					object[] attributes = type.GetCustomAttributes (false);
-					foreach (object attribute in attributes) {
-						
-						PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
-						if (mapping == null)
-							continue;
-						//Logger.Log.Debug (mapping.Keyword + " => " 
-						//		+ mapping.PropertyName
-						//		+ " is-keyword=" + mapping.IsKeyword + " (" 
-						//		+ mapping.Description + ") "
-						//		+ "(" + type.FullName + ")");
-						PropertyKeywordFu.RegisterMapping (mapping);
-					}
-				}
-			}
-		}
-
-		////////////////////////////////////////////////////////
-
 		// Scans PathFinder.SystemIndexesDir after available 
 		// system-wide indexes.
 		static void LoadSystemIndexes () 
@@ -246,12 +200,16 @@ namespace Beagle.Daemon {
 
 			int count = 0;
 
+			Queryable queryable;
 			foreach (DirectoryInfo index_dir in new DirectoryInfo (PathFinder.SystemIndexesDir).GetDirectories ()) {
 				if (! UseQueryable (index_dir.Name))
 					continue;
 				
-				if (LoadStaticQueryable (index_dir, QueryDomain.System))
+				queryable = StaticQueryable.LoadStaticQueryable (index_dir, QueryDomain.System);
+				if (queryable != null) {
+					iqueryable_to_queryable [queryable.IQueryable] = queryable;
 					count++;
+				}
 			}
 
 			Logger.Log.Info ("Found {0} system-wide indexes.", count);
@@ -265,10 +223,14 @@ namespace Beagle.Daemon {
 
 			if (UseQueryable ("static")) {
 				Logger.Log.Info ("Loading user-configured static indexes.");
-				foreach (string path in Conf.Daemon.StaticQueryables)
-					static_queryables.Add (path);
+				List<string[]> values = Conf.Daemon.GetListOptionValues (Conf.Names.StaticQueryables);
+				if (values != null) {
+					foreach (string[] path in values)
+						static_queryables.Add (path [0]);
+				}
 			}
 
+			Queryable queryable;
 			foreach (string path in static_queryables) {
 				DirectoryInfo index_dir = new DirectoryInfo (StringFu.SanitizePath (path));
 
@@ -276,45 +238,16 @@ namespace Beagle.Daemon {
 					continue;
 				
 				// FIXME: QueryDomain might be other than local
-				if (LoadStaticQueryable (index_dir, QueryDomain.Local))
+				queryable = StaticQueryable.LoadStaticQueryable (index_dir, QueryDomain.Local);
+				if (queryable != null) {
+					iqueryable_to_queryable [queryable.IQueryable] = queryable;
 					count++;
+				}
 			}
 
 			Logger.Log.Info ("Found {0} user-configured static indexes..", count);
-		}
 
-		// Instantiates and loads a StaticQueryable from an index directory
-		static private bool LoadStaticQueryable (DirectoryInfo index_dir, QueryDomain query_domain) 
-		{
-			StaticQueryable static_queryable = null;
-			
-			if (!index_dir.Exists)
-				return false;
-			
-			try {
-				static_queryable = new StaticQueryable (index_dir.Name, index_dir.FullName, true);
-			} catch (InvalidOperationException) {
-				Logger.Log.Warn ("Unable to create read-only index (likely due to index version mismatch): {0}", index_dir.FullName);
-				return false;
-			} catch (Exception e) {
-				Logger.Log.Error (e, "Caught exception while instantiating static queryable: {0}", index_dir.Name);
-				return false;
-			}
-			
-			if (static_queryable != null) {
-				QueryableFlavor flavor = new QueryableFlavor ();
-				flavor.Name = index_dir.Name;
-				flavor.Domain = query_domain;
-				
-				Queryable queryable = new Queryable (flavor, static_queryable);
-				queryables.Add (queryable);
-				
-				iqueryable_to_queryable [static_queryable] = queryable;
-
-				return true;
-			}
-
-			return false;
+			static_queryables = null;
 		}
 
 		////////////////////////////////////////////////////////
@@ -329,6 +262,8 @@ namespace Beagle.Daemon {
 			assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
 		}
 
+		////////////////////////////////////////////////////////
+
 		private static bool queryables_started = false;
 
 		static public void Start ()
@@ -347,12 +282,15 @@ namespace Beagle.Daemon {
 			
 			assemblies = null;
 
-			ReadKeywordMappings ();
+			PropertyKeywordFu.ReadKeywordMappings ();
 
 			LoadSystemIndexes ();
 			LoadStaticQueryables ();
 
-			if (indexing_delay <= 0 || Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") != null)
+			if (indexing_delay < 0)
+				return;
+
+			if (indexing_delay == 0 || Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") != null)
 				StartQueryables ();
 			else {
 				Logger.Log.Debug ("Waiting {0} seconds before starting queryables", indexing_delay);
@@ -364,9 +302,45 @@ namespace Beagle.Daemon {
 		{
 			Logger.Log.Debug ("Starting queryables");
 
-			foreach (Queryable q in queryables) {
-				Logger.Log.Info ("Starting backend: '{0}'", q.Name);
-				q.Start ();
+			ArrayList started_queryables = new ArrayList ();
+			ArrayList delayed_queryables = new ArrayList ();
+
+			ICollection queryables_to_start = Queryables;
+			int last_count;
+
+			do {
+				foreach (Queryable q in queryables_to_start) {
+					bool ready_to_start = true;
+
+					if (q.DependsOn != null && started_queryables.IndexOf (q.DependsOn) == -1) {
+						Log.Info ("Delaying backend '{0}' until after backend '{1}'",
+							  q.Name, q.DependsOn);
+						ready_to_start = false;
+						delayed_queryables.Add (q);
+					}
+
+					if (! ready_to_start)
+						continue;
+
+					Logger.Log.Info ("Starting backend: '{0}'", q.Name);
+					q.Start ();
+
+					started_queryables.Add (q.Name);
+				}
+
+				last_count = queryables_to_start.Count;
+
+				queryables_to_start = delayed_queryables;
+				delayed_queryables = new ArrayList ();
+			} while (queryables_to_start.Count > 0 && queryables_to_start.Count != last_count);
+
+			if (queryables_to_start.Count > 0) {
+				Log.Info ("Unable to start backends due to missing or circular dependencies:");
+
+				foreach (Queryable q in queryables_to_start) {
+					Log.Info ("  - {0}", q.Name);
+					iqueryable_to_queryable.Remove (q.IQueryable);
+				}
 			}
 
 			queryables_started = true;
@@ -374,7 +348,9 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
-		static public string ListBackends ()
+		////////////////////////////////////////////////////////
+
+		public static string ListBackends ()
 		{
 			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
 
@@ -386,8 +362,13 @@ namespace Beagle.Daemon {
 
 			foreach (Assembly assembly in assemblies) {
 				foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (IQueryableTypesAttribute))) {
-					foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor)))
-						ret += String.Format (" - {0}\n", flavor.Name);
+					foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor))) {
+						ret += String.Format (" - {0}", flavor.Name);
+						if (flavor.DependsOn != null)
+							ret += String.Format (" (depends on {0})", flavor.DependsOn);
+
+						ret += "\n";
+					}
 				}
 			}
 			
@@ -404,7 +385,7 @@ namespace Beagle.Daemon {
 
 		static public Queryable GetQueryable (string name)
 		{
-			foreach (Queryable q in queryables) {
+			foreach (Queryable q in Queryables) {
 				if (q.Name == name)
 					return q;
 			}
@@ -465,9 +446,13 @@ namespace Beagle.Daemon {
 					       QueryResult          result,
 					       IQueryableChangeData change_data)
 		{
-			if (queryable.AcceptQuery (query)) {
-				QueryClosure qc = new QueryClosure (queryable, query, result, change_data);
-				result.AttachWorker (qc);
+			try {
+				if (queryable.AcceptQuery (query)) {
+					QueryClosure qc = new QueryClosure (queryable, query, result, change_data);
+					result.AttachWorker (qc);
+				}
+			} catch (Exception ex) {
+				Logger.Log.Warn (ex, "Caught exception calling DoOneQuery on '{0}'", queryable.Name);
 			}
 		}
 
@@ -517,7 +502,7 @@ namespace Beagle.Daemon {
 					continue;
 				if (sb.Length > 0)
 					sb.Append (' ');
-				sb.Append (LuceneCommon.Stem (split [i]));
+				sb.Append (LuceneCommon.Stem (split [i].ToLower ()));
 			}
 			response.StemmedText.Add (sb.ToString ());
 		}
@@ -567,8 +552,7 @@ namespace Beagle.Daemon {
 			return search_term_response;
 		}
 
-		static private void QueryEachQueryable (Query       query,
-							QueryResult result)
+		static private void QueryEachQueryable (Query query, QueryResult result)
 		{
 			// The extra pair of calls to WorkerStart/WorkerFinished ensures:
 			// (1) that the QueryResult will fire the StartedEvent
@@ -581,13 +565,19 @@ namespace Beagle.Daemon {
 
 			if (! result.WorkerStart (dummy_worker))
 				return;
-			
-			foreach (Queryable queryable in queryables)
-				DoOneQuery (queryable, query, result, null);
+
+			try {
+				iqueryable_lock.AcquireReaderLock (System.Threading.Timeout.Infinite);
+
+				foreach (Queryable queryable in Queryables)
+					DoOneQuery (queryable, query, result, null);
+			} finally {
+				iqueryable_lock.ReleaseReaderLock ();
+			}
 			
 			result.WorkerFinished (dummy_worker);
 		}
-		
+
 		static public void DoQueryLocal (Query       query,
 						 QueryResult result)
 		{
@@ -613,12 +603,67 @@ namespace Beagle.Daemon {
 			QueryEachQueryable (query, result);
 		}
 
+#if ENABLE_RDF_ADAPTER
+		static public ArrayList DoRDFQuery (RDFQuery query)
+		{
+			ArrayList all_results = new ArrayList ();
+
+			try {
+				iqueryable_lock.AcquireReaderLock (System.Threading.Timeout.Infinite);
+
+				foreach (Queryable q in Queryables) {
+					if (! q.AcceptQuery (query))
+						continue;
+
+					ICollection results = q.DoRDFQuery (query);
+					if (results == null || results.Count == 0)
+						continue;
+					all_results.AddRange (results);
+				}
+			} finally {
+				iqueryable_lock.ReleaseReaderLock ();
+			}
+
+			return all_results;
+		}
+#endif
+
+		////////////////////////////////////////////////////////
+
+		static public int DoCountMatchQuery (CountMatchQuery query)
+		{
+			DehumanizeQuery (query);
+
+			int num_matches = 0;
+
+			try {
+				iqueryable_lock.AcquireReaderLock (System.Threading.Timeout.Infinite);
+
+				foreach (Queryable q in Queryables) {
+					if (! q.AcceptQuery (query))
+						continue;
+
+					num_matches += q.DoCountMatchQuery (query);
+				}
+			} finally {
+				iqueryable_lock.ReleaseReaderLock ();
+			}
+
+			return num_matches;
+		}
+
 		////////////////////////////////////////////////////////
 
 		static public IEnumerable GetIndexInformation ()
 		{
-			foreach (Queryable q in queryables)
-				yield return q.GetQueryableStatus ();
+			try {
+				iqueryable_lock.AcquireReaderLock (System.Threading.Timeout.Infinite);
+
+				foreach (Queryable q in Queryables)
+					yield return q.GetQueryableStatus ();
+			} finally {
+				iqueryable_lock.ReleaseReaderLock ();
+			}
 		}
 
 		////////////////////////////////////////////////////////
@@ -631,17 +676,156 @@ namespace Beagle.Daemon {
 				if (! queryables_started)
 					return true;
 
-				foreach (Queryable q in queryables) {
-					QueryableStatus status = q.GetQueryableStatus ();
+				try {
+					iqueryable_lock.AcquireReaderLock (System.Threading.Timeout.Infinite);
 
-					if (status == null)
-						return false;
+					foreach (Queryable q in Queryables) {
+						QueryableStatus status = q.GetQueryableStatus ();
 
-					if (status.IsIndexing)
-						return true;
+						if (status == null)
+							return false;
+
+						if (status.IsIndexing)
+							return true;
+					}
+
+					return false;
+				} finally {
+					iqueryable_lock.ReleaseReaderLock ();
 				}
+			}
+		}
 
-				return false;
+		/////////////////////////////////////////////////////////
+
+		// Removable Index related stuff
+
+		private static Dictionary<string, Queryable> removable_queryables = new Dictionary<string, Queryable> (4); // small number
+
+		internal static ResponseMessage HandleRemovableIndexRequest (bool to_mount, string index_dir, string mnt_dir)
+		{
+			lock (removable_queryables) {
+				if (to_mount)
+					return AddRemovableIndex (index_dir, mnt_dir);
+				else
+					return RemoveRemovableIndex (index_dir, mnt_dir);
+			}
+		}
+
+		private static ResponseMessage AddRemovableIndex (string path, string mnt_dir)
+		{
+			DirectoryInfo index_dir = new DirectoryInfo (StringFu.SanitizePath (path));
+			if (! index_dir.Exists) {
+				ErrorResponse msg;
+				msg = new ErrorResponse ();
+				msg.ErrorMessage = "Adding removable index failed";
+				msg.Details = String.Format ("'{0}' does not exist.", path);
+				return msg;
+			}
+
+			// Allow late loading of mount dir ?
+			mnt_dir = StringFu.SanitizePath (mnt_dir);
+			if (! Directory.Exists (mnt_dir)) {
+				ErrorResponse msg;
+				msg = new ErrorResponse ();
+				msg.ErrorMessage = "Adding removable index failed";
+				msg.Details = String.Format ("Mount directory '{0}' does not exist.", mnt_dir);
+				return msg;
+			}
+
+			if (removable_queryables.ContainsKey (path)) {
+				ErrorResponse msg;
+				msg = new ErrorResponse ();
+				msg.ErrorMessage = "Adding removable index failed";
+				msg.Details = String.Format ("'{0}' already added.", path);
+				return msg;
+			}
+
+			Queryable removable_queryable = null;
+
+			try {
+				iqueryable_lock.AcquireWriterLock (System.Threading.Timeout.Infinite);
+				removable_queryable = StaticQueryable.LoadRemovableQueryable (index_dir, mnt_dir);
+			} finally {
+				iqueryable_lock.ReleaseWriterLock ();
+			}
+
+			if (removable_queryable == null)
+				return new ErrorResponse ("Adding removable index failed");
+
+			iqueryable_to_queryable [removable_queryable.IQueryable] = removable_queryable;
+			removable_queryables [path] = removable_queryable;
+
+			RemovableIndexResponse resp = new RemovableIndexResponse ();
+			resp.Source = removable_queryable.Name;
+			Log.Info ("Adding removable index '{0}' from {1}", resp.Source, path);
+			return resp;
+		}
+
+		private static ResponseMessage RemoveRemovableIndex (string path, string mnt_dir)
+		{
+			if (! removable_queryables.ContainsKey (path)) {
+				ErrorResponse msg;
+				msg = new ErrorResponse ();
+				msg.ErrorMessage = "Removing removable-index failed";
+				msg.Details = String.Format ("'{0}' was not added.", path);
+				return msg;
+			}
+
+			Queryable removable_queryable = removable_queryables [path];
+
+			// Assert
+			if (removable_queryable == null ||
+			    ! (removable_queryable.IQueryable is StaticQueryable)) {
+				ErrorResponse msg = new ErrorResponse ("Removing removable-index failed");
+				return msg;
+			}
+
+			StaticQueryable static_queryable = (StaticQueryable) removable_queryable.IQueryable;
+			if (static_queryable.RemovableMountDir != mnt_dir) {
+				ErrorResponse msg;
+				msg = new ErrorResponse ();
+				msg.ErrorMessage = "Removing removable-index failed";
+				msg.Details = String.Format ("No index mounted at {0}.", mnt_dir);
+				return msg;
+			}
+
+			static_queryable.Close ();
+
+			removable_queryables.Remove (path);
+
+			try {
+				iqueryable_lock.AcquireWriterLock (System.Threading.Timeout.Infinite);
+				iqueryable_to_queryable [removable_queryable.IQueryable] = null;
+				iqueryable_to_queryable.Remove (removable_queryable.IQueryable);
+			} finally {
+				iqueryable_lock.ReleaseWriterLock ();
+			}
+
+			RemovableIndexResponse resp = new RemovableIndexResponse ();
+			resp.Source = removable_queryable.Name;
+			Log.Info ("Removed removable-index '{0}' at {1}", removable_queryable.Name, path);
+			return resp;
+		}
+
+		/////////////////////////////////////////////////////////
+
+		// Various debug hooks; especially persistant state information and memory issues
+
+		public static void DebugHook ()
+		{
+			Log.Debug ("Debughook called:");
+			LuceneQueryable l;
+
+			foreach (Queryable q in Queryables) {
+				l = q.IQueryable as LuceneQueryable;
+				if (l == null)
+					continue;
+				try {
+					l.DebugHook ();
+				} catch (Exception e) {
+					Log.Debug (e, "Exception during Debughook for {0}", q.Name);
+				}
 			}
 		}
 	}
