@@ -61,12 +61,15 @@ namespace Beagle.Daemon {
 		{
 			Hit requested = (Hit) items [path];
 
-			if (requested == null)
+			if (requested == null || ! requested.Uri.IsFile) {
+				context.Response.StatusCode = 404;
+				context.Response.Close ();
 				return;
+			}
 
 			//Logger.Log.Debug ("httpitemhandler: requested: {0}", path);
 			context.Response.ContentType = requested.MimeType;
-			
+
 			// FIXME: We can only handle files for now
 			/*if (requested ["Type"] != "File") {
 			  Logger.Log.Debug ("httpitemhandler: can only serve files");
@@ -97,12 +100,16 @@ namespace Beagle.Daemon {
 		private HttpItemHandler item_handler = null;
 		private System.Timers.Timer keepalive_timer = null;
 		private System.Guid id = Guid.Empty;
+		private string network_node = null;
 
 		public HttpConnectionHandler (System.Guid guid, HttpListenerContext context, HttpItemHandler item_handler)
 		{
 			this.id = guid;
 			this.context = context;
 			this.item_handler = item_handler;
+
+			Config config = Conf.Get (Conf.Names.NetworkingConfig);
+			network_node = config.GetOption ("ServiceName", String.Empty);
 		}
 
 		internal System.Guid Guid {
@@ -165,7 +172,7 @@ namespace Beagle.Daemon {
 
 				if (bytes_read > 0) {
 					// 0xff signifies end of message
-					end_index = ArrayFu.IndexOfByte (network_data, (byte) 0xff);
+					end_index = Array.IndexOf<byte> (network_data, (byte) 0xff);
 
 					buffer_stream.Write (network_data, 0,
 							     end_index == -1 ? bytes_read : end_index);
@@ -174,8 +181,13 @@ namespace Beagle.Daemon {
 			
 			//Logger.Log.Debug ("HTTP Server: Handling received request message");
 			
-			buffer_stream.Seek (0, SeekOrigin.Begin);
-			base.HandleConnection (buffer_stream);
+                        // The 0xff bytes from a remote beagled comes in a separate http request
+                        // and causes havoc by creating empty messages. HTTP streams do not behave
+                        // like a continous stream like UnixStream
+                        if (total_bytes > 0) {
+				buffer_stream.Seek (0, SeekOrigin.Begin);
+				base.HandleConnection (buffer_stream);
+			}
 			
 			Server.MarkHandlerAsKilled (this);
 			Shutdown.WorkerFinished (context.Request.InputStream);
@@ -253,17 +265,30 @@ namespace Beagle.Daemon {
 			if (response == null)
 				return;
 
+			// Change the Hit source and add a property with the name of this remote node
 			foreach (Hit hit in response.Hits) {
-				hit.Uri = new System.Uri (context.Request.Url.ToString () + id.ToString ());
-				hit ["beagle:Source"] = "Network";
+				//hit.Uri = new System.Uri (context.Request.Url.ToString () + id.ToString ());
+				IList new_props = new Property [2];
+				new_props [0] = Property.NewKeyword ("beagle:OrigSource", hit ["beagle:Source"]);
+				new_props [1] = Property.NewKeyword ("beagle:NetworkNode", network_node);
+				hit.AddProperty (new_props);
 
-				item_handler.RegisterHit (hit);
-			}	
+                                // Need to replace the existing beagle:Source property
+                                foreach (Property prop in hit.Properties) {
+                                        if (prop.Key == "beagle:Source") {
+                                                prop.Value = "NetworkServices";
+                                                break;
+                                        }
+                                }
+
+				//FIXME Retrieving the files for a remote hit is disabled for now
+				//item_handler.RegisterHit (hit);
+			}
 		}
 
 		public override bool SendResponse (ResponseMessage response)
-		{	
-			//TransformResponse (ref response); Disable for now
+		{
+			TransformResponse (ref response);
 
 			bool r = false;
 
@@ -294,12 +319,14 @@ namespace Beagle.Daemon {
 		public UnixConnectionHandler (UnixClient client)
 		{
 			this.client = client;
+			this.client.SendBufferSize = 4096;
+			this.client.ReceiveBufferSize = 4096;
 		}
 
 		public override bool SendResponse(ResponseMessage response)
 		{
 			bool result = false;
-			
+
 			lock (this.client_lock) {
 				if (this.client == null) 
 					return false;
@@ -377,7 +404,7 @@ namespace Beagle.Daemon {
 
 				if (bytes_read > 0) {
 					// 0xff signifies end of message
-					end_index = ArrayFu.IndexOfByte (network_data, (byte) 0xff);
+					end_index = Array.IndexOf<byte> (network_data, (byte) 0xff);
 
 					buffer_stream.Write (network_data, 0,
 							     end_index == -1 ? bytes_read : end_index);
@@ -409,6 +436,8 @@ namespace Beagle.Daemon {
 			Shutdown.WorkerFinished (network_data);
 		}
 
+		private bool closed = false;
+
 		public override void Close ()
 		{
 			CancelIfBlocking ();
@@ -417,17 +446,24 @@ namespace Beagle.Daemon {
 			// grab the lock here and close the underlying
 			// UnixClient, or else we'd deadlock between here and
 			// the Read() in HandleConnection()
+
+			// Do this in a lock since Close() should be thread-safe (can be called from AsyncCallback handler)
 			lock (this.client_lock) {
+				if (closed)
+					return;
+
 				if (this.client != null) {
 					this.client.Close ();
 					this.client = null;
 				}
-			}
 
-			if (this.executor != null) {
-				this.executor.Cleanup ();
-				this.executor.AsyncResponseEvent -= OnAsyncResponse;
-				this.executor = null;
+				if (this.executor != null) {
+					this.executor.Cleanup ();
+					this.executor.AsyncResponseEvent -= OnAsyncResponse;
+					this.executor = null;
+				}
+
+			    closed = true;
 			}
 
 			Server.RunGC ();
@@ -439,8 +475,10 @@ namespace Beagle.Daemon {
 
 			try {
 				bytes_read = this.client.GetStream ().EndRead (ar);
-			} catch (SocketException) {
-			} catch (IOException) { }
+			} catch (IOException e) {
+				if (! (e.InnerException is SocketException))
+					throw e;
+			} catch (ObjectDisposedException) { }
 
 			if (bytes_read == 0)
 				Close ();
@@ -455,8 +493,26 @@ namespace Beagle.Daemon {
 				return;
 			}
 
-			this.client.GetStream ().BeginRead (new byte[1024], 0, 1024,
-							    new AsyncCallback (WatchCallback), null);
+			try {
+				this.client.GetStream ().BeginRead (new byte[1024], 0, 1024,
+								    new AsyncCallback (WatchCallback), null);
+			} catch (ObjectDisposedException) {
+				Log.Debug ("Network stream closed; closing connection at this end");
+				this.Close ();
+				return;
+			} catch (IOException e) {
+				if (e.InnerException is SocketException) {
+					Log.Debug (e.InnerException, "Socket exception while setting up watch");
+					this.Close ();
+					return;
+				} else
+					throw e;
+			} catch (System.ArgumentException e) {
+				// https://bugzilla.novell.com/show_bug.cgi?id=371923
+				Log.Debug (e, "Possibly socket is already closed");
+				this.Close ();
+				return;
+			}
 		}
 	}	
 
@@ -839,7 +895,7 @@ namespace Beagle.Daemon {
 					Guid guid = Guid.NewGuid ();
 					HttpItemHandler item_handler = new HttpItemHandler ();
 					item_handlers [guid] = item_handler;
-					
+
 					ConnectionHandler handler = new HttpConnectionHandler (guid, context, item_handler);
 
 					lock (live_handlers)
