@@ -27,7 +27,9 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
+using System.Text;
 using System.Threading;
 
 using Mono.Data.Sqlite;
@@ -46,14 +48,18 @@ namespace Beagle.Daemon {
 	//
 	// See http://bugzilla.gnome.org/show_bug.cgi?329022
 
-	public class TextCache {
+	public class TextCache : IDisposable {
 
 		private const string SELF_CACHE_TAG = "*self*";
 		private const string BLOB_TAG = "*blob*";
-		public SqliteCommand InsertCommand;
-		public SqliteCommand LookupPathCommand;
-		public SqliteCommand LookupDataCommand;
-		public SqliteCommand DeleteCommand;
+		private SqliteCommand InsertCommand;
+		private SqliteCommand LookupPathCommand;
+		private SqliteCommand LookupDataCommand;
+		private SqliteCommand DeleteCommand;
+#if ENABLE_RDF_ADAPTER
+		private SqliteCommand UpdateLinksCommand;
+		private SqliteCommand LookupLinksCommand;
+#endif
 		private string text_cache_dir;
 		internal string TextCacheDir {
 			get { return text_cache_dir; }
@@ -163,12 +169,21 @@ namespace Beagle.Daemon {
 
 				// Database schema: uri, filename, data
 				SqliteUtils.DoNonQuery (connection,
-							"CREATE TABLE textcache_data (            " +
+							"CREATE TABLE textcache_data (     " +
 							"  uri      TEXT UNIQUE NOT NULL,  " +
 							"  filename TEXT NOT NULL,         " +
-							"  data     BLOB                     " +
+							"  data     BLOB                   " +
 							")");
 			}
+#if ENABLE_RDF_ADAPTER
+			try {
+				SqliteUtils.DoNonQuery (connection,
+							"CREATE TABLE IF NOT EXISTS links_data (  " +
+							"  uri TEXT UNIQUE NOT NULL,		  " +
+							"  links TEXT				  " +
+							")");
+			} catch (SqliteException) { }
+#endif
 			this.InitCommands ();
 		}
 
@@ -182,7 +197,16 @@ namespace Beagle.Daemon {
 			LookupDataCommand.CommandText = "SELECT filename, data FROM textcache_data WHERE uri=@uri";
 			DeleteCommand = new SqliteCommand (this.connection);
 			DeleteCommand.CommandText = "DELETE FROM textcache_data WHERE uri=@uri";
+
+#if ENABLE_RDF_ADAPTER
+			UpdateLinksCommand = new SqliteCommand (this.connection);
+			UpdateLinksCommand.CommandText = "INSERT OR REPLACE INTO links_data (uri, links) VALUES (@uri, @links);";
+
+			LookupLinksCommand = new SqliteCommand (this.connection);
+			LookupLinksCommand.CommandText = "SELECT links FROM links_data WHERE uri=@uri";
+#endif
 		}
+
 		private SqliteConnection Open (string db_filename)
 		{
 			SqliteConnection connection = new SqliteConnection ();
@@ -191,6 +215,20 @@ namespace Beagle.Daemon {
 			return connection;
 		}
 
+		public void Dispose ()
+		{
+			InsertCommand.Dispose ();
+			LookupPathCommand.Dispose ();
+			LookupDataCommand.Dispose ();
+			DeleteCommand.Dispose ();
+#if ENABLE_RDF_ADAPTER
+			UpdateLinksCommand.Dispose ();
+			LookupLinksCommand.Dispose ();
+#endif
+			connection.Dispose ();
+			connection = null;
+		}
+
 		private static string UriToString (Uri uri)
 		{
 			return UriFu.UriToEscapedString (uri).Replace ("'", "''");
@@ -233,7 +271,7 @@ namespace Beagle.Daemon {
 				if (SqliteUtils.ReadOrWait (reader))
 					path = reader.GetString (0);
 			}
-			
+
 			return path;
 		}
 
@@ -342,7 +380,10 @@ namespace Beagle.Daemon {
 			
 				if (! world_readable) {
 					// Make files only readable by the owner.
-					Mono.Unix.Native.Syscall.chmod (path, (Mono.Unix.Native.FilePermissions) 384);
+					Mono.Unix.Native.Syscall.chmod (
+						path,
+						Mono.Unix.Native.FilePermissions.S_IRUSR |
+						Mono.Unix.Native.FilePermissions.S_IWUSR);
 				}
 
 				stream = file_stream;
@@ -458,7 +499,7 @@ namespace Beagle.Daemon {
 
 			lock (connection) {
 				
-				LookupDataCommand.Parameters.AddWithValue("@uri",UriToString (uri));
+				LookupDataCommand.Parameters.AddWithValue ("@uri", UriToString (uri));
 				using (SqliteDataReader reader = SqliteUtils.ExecuteReaderOrWait (LookupDataCommand)) {
 					if (! SqliteUtils.ReadOrWait (reader)) {
 						if (self_cache)
@@ -466,9 +507,9 @@ namespace Beagle.Daemon {
 						return null;
 					}
 
-				filename = reader.GetString (0);
-				if (! reader.IsDBNull (1))
-					blob = reader.GetValue (1) as byte [];
+					filename = reader.GetString (0);
+					if (! reader.IsDBNull (1))
+						blob = reader.GetValue (1) as byte [];
 				}
 
 			}
@@ -518,6 +559,55 @@ namespace Beagle.Daemon {
 			}
 		}
 
+#if ENABLE_RDF_ADAPTER
+		public void AddLinks (Uri uri, IList<string> links)
+		{
+			lock (connection) {
+				MaybeStartTransaction_Unlocked ();
+				UpdateLinksCommand.Parameters.AddWithValue("@uri", UriToString (uri));
+				UpdateLinksCommand.Parameters.AddWithValue("@links", GetLinksText (links));
+				SqliteUtils.DoNonQuery (UpdateLinksCommand);
+			}
+		}
+
+		private string GetLinksText (IList<string> links)
+		{
+			if (links == null || links.Count == 0)
+				return String.Empty;
+
+			StringBuilder sb = new StringBuilder ();
+			foreach (string s in links) {
+				sb.Append (s);
+				sb.Append (' ');
+			}
+
+			return sb.ToString ();
+		}
+
+		public IList<string> GetLinks (Uri uri)
+		{
+			string links_text = null;
+			List<string> links = null;
+
+			lock (connection) {
+				LookupLinksCommand.Parameters.AddWithValue ("@uri", UriToString (uri));
+				using (SqliteDataReader reader = SqliteUtils.ExecuteReaderOrWait (LookupLinksCommand)) {
+					if (! SqliteUtils.ReadOrWait (reader))
+						return null;
+
+					links_text = reader.GetString (0);
+				}
+			}
+
+			if (String.IsNullOrEmpty (links_text))
+				return null;
+
+			return links_text.Split (links_separator, StringSplitOptions.RemoveEmptyEntries);
+		}
+
+		static readonly char[] links_separator = new char[] {' '};
+#endif
+
 		private void MaybeStartTransaction_Unlocked ()
 		{
 			if (transaction_state == TransactionState.Requested)
